/***************************************************************************
                          _aacodes.cpp  -  description
                             -------------------
    begin                : Tue Jul 8 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "_aacodes.h"

using namespace N_AminoAcid;

S_AACodes N_AminoAcid::g_AACodes = S_AACodes ();


/***************************************************************************
 *                                                                         *
 *   S_AACodes                                                             *
 *                                                                         *
 ***************************************************************************/

S_AACodes::
S_AACodes ()
{
    m_Known1lCodes_map ['A'] = ALANINE;
    m_Known1lCodes_map ['R'] = ARGININE;
    m_Known1lCodes_map ['N'] = ASPARAGINE;
    m_Known1lCodes_map ['D'] = ASPARTATE;
    m_Known1lCodes_map ['B'] = ASPARAGINE_OR_ASPARTATE;
    m_Known1lCodes_map ['C'] = CYSTEINE;
    m_Known1lCodes_map ['Q'] = GLUTAMINE;
    m_Known1lCodes_map ['E'] = GLUTAMATE;
    m_Known1lCodes_map ['Z'] = GLUTAMINE_OR_GLUTAMATE;
    m_Known1lCodes_map ['G'] = GLYCINE;
    m_Known1lCodes_map ['H'] = HISTIDINE;
    m_Known1lCodes_map ['I'] = ISOLEUCINE;
    m_Known1lCodes_map ['L'] = LEUCINE;
    m_Known1lCodes_map ['K'] = LYSINE;
    m_Known1lCodes_map ['M'] = METHIONINE;
    m_Known1lCodes_map ['F'] = PHENYLALANINE;
    m_Known1lCodes_map ['P'] = PROLINE;
    m_Known1lCodes_map ['U'] = SELENOCYSTEINE;
    m_Known1lCodes_map ['S'] = SERINE;
    m_Known1lCodes_map ['T'] = THREONINE;
    m_Known1lCodes_map ['W'] = TRYPTOPHANE;
    m_Known1lCodes_map ['Y'] = TYROSINE;
    m_Known1lCodes_map ['V'] = VALINE;
    m_Known1lCodes_map ['X'] = ANY;
    m_Known1lCodes_map ['*'] = TRANSLATION_STOP;
    m_Known1lCodes_map ['.'] = GAP;
    m_Known1lCodes_map ['-'] = GAP;
    m_Known1lCodes_map ['~'] = GAP;
    m_Known1lCodes_map ['_'] = GAP;
    m_Known1lCodes_map ['?'] = UNKNOWN;

    m_Known3lCodes_map ["ALA"] = ALANINE;
    m_Known3lCodes_map ["ARG"] = ARGININE;
    m_Known3lCodes_map ["ASN"] = ASPARAGINE;
    m_Known3lCodes_map ["ASP"] = ASPARTATE;
    m_Known3lCodes_map ["ASX"] = ASPARAGINE_OR_ASPARTATE;
    m_Known3lCodes_map ["CYS"] = CYSTEINE;
    m_Known3lCodes_map ["GLN"] = GLUTAMINE;
    m_Known3lCodes_map ["GLU"] = GLUTAMATE;
    m_Known3lCodes_map ["GLX"] = GLUTAMINE_OR_GLUTAMATE;
    m_Known3lCodes_map ["GLY"] = GLYCINE;
    m_Known3lCodes_map ["HIS"] = HISTIDINE;
    m_Known3lCodes_map ["HSE"] = HISTIDINE;
    m_Known3lCodes_map ["HSP"] = HISTIDINE;
    m_Known3lCodes_map ["HSD"] = HISTIDINE;
    m_Known3lCodes_map ["ILE"] = ISOLEUCINE;
    m_Known3lCodes_map ["LEU"] = LEUCINE;
    m_Known3lCodes_map ["LYS"] = LYSINE;
    m_Known3lCodes_map ["MET"] = METHIONINE;
    m_Known3lCodes_map ["PHE"] = PHENYLALANINE;
    m_Known3lCodes_map ["PRO"] = PROLINE;
    m_Known3lCodes_map ["SEL"] = SELENOCYSTEINE;
    m_Known3lCodes_map ["SER"] = SERINE;
    m_Known3lCodes_map ["THR"] = THREONINE;
    m_Known3lCodes_map ["TRP"] = TRYPTOPHANE;
    m_Known3lCodes_map ["TYR"] = TYROSINE;
    m_Known3lCodes_map ["VAL"] = VALINE;
    m_Known3lCodes_map ["***"] = ANY;
    m_Known3lCodes_map ["ANY"] = ANY;
    m_Known3lCodes_map ["TGA"] = TRANSLATION_STOP;
    m_Known3lCodes_map ["TAG"] = TRANSLATION_STOP;
    m_Known3lCodes_map ["TAA"] = TRANSLATION_STOP;
    m_Known3lCodes_map ["---"] = GAP;
    m_Known3lCodes_map ["GAP"] = GAP;
    m_Known3lCodes_map ["UNK"] = UNKNOWN;

    m_KnownStrCodes_map ["ALANINE"] = ALANINE;
    m_KnownStrCodes_map ["ARGININE"] = ARGININE;
    m_KnownStrCodes_map ["ASPARAGINE"] = ASPARAGINE;
    m_KnownStrCodes_map ["ASPARTIC_ACID"] = ASPARTATE;
    m_KnownStrCodes_map ["ASPARTATE"] = ASPARTATE;
    m_KnownStrCodes_map ["ASPARAGINE_OR_ASPARTIC_ACID"] = ASPARAGINE_OR_ASPARTATE;
    m_KnownStrCodes_map ["ASPARAGINE_OR_ASPARTATE"] = ASPARAGINE_OR_ASPARTATE;
    m_KnownStrCodes_map ["CYSTEINE"] = CYSTEINE;
    m_KnownStrCodes_map ["GLUTAMINE"] = GLUTAMINE;
    m_KnownStrCodes_map ["GLUTAMIC_ACID"] = GLUTAMATE;
    m_KnownStrCodes_map ["GLUTAMATE"] = GLUTAMATE;
    m_KnownStrCodes_map ["GLUTAMINE_OR_GLUTAMIC_ACID"] = GLUTAMINE_OR_GLUTAMATE;
    m_KnownStrCodes_map ["GLUTAMINE_OR_GLUTAMATE"] = GLUTAMINE_OR_GLUTAMATE;
    m_KnownStrCodes_map ["GLYCINE"] = GLYCINE;
    m_KnownStrCodes_map ["HISTIDINE"] = HISTIDINE;
    m_KnownStrCodes_map ["ISOLEUCINE"] = ISOLEUCINE;
    m_KnownStrCodes_map ["LEUCINE"] = LEUCINE;
    m_KnownStrCodes_map ["LYSINE"] = LYSINE;
    m_KnownStrCodes_map ["METHIONINE"] = METHIONINE;
    m_KnownStrCodes_map ["PHENYLALANINE"] = PHENYLALANINE;
    m_KnownStrCodes_map ["PROLINE"] = PROLINE;
    m_KnownStrCodes_map ["SELENOCYSTEINE"] = SELENOCYSTEINE;
    m_KnownStrCodes_map ["SERINE"] = SERINE;
    m_KnownStrCodes_map ["THREONINE"] = THREONINE;
    m_KnownStrCodes_map ["TRYPTOPHANE"] = TRYPTOPHANE;
    m_KnownStrCodes_map ["TYROSINE"] = TYROSINE;
    m_KnownStrCodes_map ["VALINE"] = VALINE;
    m_KnownStrCodes_map ["ANY"] = ANY;
    m_KnownStrCodes_map ["TRANSLATION_STOP"] = TRANSLATION_STOP;
    m_KnownStrCodes_map ["GAP"] = GAP;
    m_KnownStrCodes_map ["UNKNOWN"] = UNKNOWN;


    /***************************************************************************                                                                         *
        default 1-letter code
        */

    m_Default1lCodes_map [ALANINE] = 'A';
    m_Default1lCodes_map [ARGININE] = 'R';
    m_Default1lCodes_map [ASPARAGINE] = 'N';
    m_Default1lCodes_map [ASPARTATE] = 'D';
    m_Default1lCodes_map [ASPARAGINE_OR_ASPARTATE] = 'B';
    m_Default1lCodes_map [CYSTEINE] = 'C';
    m_Default1lCodes_map [GLUTAMINE] = 'Q';
    m_Default1lCodes_map [GLUTAMATE] = 'E';
    m_Default1lCodes_map [GLUTAMINE_OR_GLUTAMATE] = 'Z';
    m_Default1lCodes_map [GLYCINE] = 'G';
    m_Default1lCodes_map [HISTIDINE] = 'H';
    m_Default1lCodes_map [ISOLEUCINE] = 'I';
    m_Default1lCodes_map [LEUCINE] = 'L';
    m_Default1lCodes_map [LYSINE] = 'K';
    m_Default1lCodes_map [METHIONINE] = 'M';
    m_Default1lCodes_map [PHENYLALANINE] = 'F';
    m_Default1lCodes_map [PROLINE] = 'P';
    m_Default1lCodes_map [SELENOCYSTEINE] = 'U';
    m_Default1lCodes_map [SERINE] = 'S';
    m_Default1lCodes_map [THREONINE] = 'T';
    m_Default1lCodes_map [TRYPTOPHANE] = 'W';
    m_Default1lCodes_map [TYROSINE] = 'Y';
    m_Default1lCodes_map [VALINE] = 'V';
    m_Default1lCodes_map [ANY] = 'X';
    m_Default1lCodes_map [TRANSLATION_STOP] = '*';
    m_Default1lCodes_map [GAP] = '.';
    m_Default1lCodes_map [UNKNOWN] = '?';


    /***************************************************************************                                                                         *
        default 3-letter code
        */


    m_Default3lCodes_map [ALANINE] = "ALA";
    m_Default3lCodes_map [ARGININE] = "ARG";
    m_Default3lCodes_map [ASPARAGINE] = "ASN";
    m_Default3lCodes_map [ASPARTATE] = "ASP";
    m_Default3lCodes_map [ASPARAGINE_OR_ASPARTATE] = "ASX";
    m_Default3lCodes_map [CYSTEINE] = "CYS";
    m_Default3lCodes_map [GLUTAMINE] = "GLN";
    m_Default3lCodes_map [GLUTAMATE] = "GLU";
    m_Default3lCodes_map [GLUTAMINE_OR_GLUTAMATE] = "GLX";
    m_Default3lCodes_map [GLYCINE] = "GLY";
    m_Default3lCodes_map [HISTIDINE] = "HIS";
    m_Default3lCodes_map [ISOLEUCINE] = "ILE";
    m_Default3lCodes_map [LEUCINE] = "LEU";
    m_Default3lCodes_map [LYSINE] = "LYS";
    m_Default3lCodes_map [METHIONINE] = "MET";
    m_Default3lCodes_map [PHENYLALANINE] = "PHE";
    m_Default3lCodes_map [PROLINE] = "PRO";
    m_Default3lCodes_map [SELENOCYSTEINE] = "SEL";
    m_Default3lCodes_map [SERINE] = "SER";
    m_Default3lCodes_map [THREONINE] = "THR";
    m_Default3lCodes_map [TRYPTOPHANE] = "TRP";
    m_Default3lCodes_map [TYROSINE] = "TYR";
    m_Default3lCodes_map [VALINE] = "VAL";
    m_Default3lCodes_map [ANY] = "***";
    m_Default3lCodes_map [TRANSLATION_STOP] = "TGA";
    m_Default3lCodes_map [GAP] = "GAP";
    m_Default3lCodes_map [UNKNOWN] = "UNK";

    /***************************************************************************                                                                         *
        default string code
        */


    m_DefaultStrCodes_map [ALANINE] = "ALANINE";
    m_DefaultStrCodes_map [ARGININE] = "ARGININE";
    m_DefaultStrCodes_map [ASPARAGINE] = "ASPARAGINE";
    m_DefaultStrCodes_map [ASPARTATE] = "ASPARTATE";
    m_DefaultStrCodes_map [ASPARAGINE_OR_ASPARTATE] = "ASPARAGINE_OR_ASPARTATE";
    m_DefaultStrCodes_map [CYSTEINE] = "CYSTEINE";
    m_DefaultStrCodes_map [GLUTAMINE] = "GLUTAMINE";
    m_DefaultStrCodes_map [GLUTAMATE] = "GLUTAMATE";
    m_DefaultStrCodes_map [GLUTAMINE_OR_GLUTAMATE] = "GLUTAMINE_OR_GLUTAMATE";
    m_DefaultStrCodes_map [GLYCINE] = "GLYCINE";
    m_DefaultStrCodes_map [HISTIDINE] = "HISTIDINE";
    m_DefaultStrCodes_map [ISOLEUCINE] = "ISOLEUCINE";
    m_DefaultStrCodes_map [LEUCINE] = "LEUCINE";
    m_DefaultStrCodes_map [LYSINE] = "LYSINE";
    m_DefaultStrCodes_map [METHIONINE] = "METHIONINE";
    m_DefaultStrCodes_map [PHENYLALANINE] = "PHENYLALANINE";
    m_DefaultStrCodes_map [PROLINE] = "PROLINE";
    m_DefaultStrCodes_map [SELENOCYSTEINE] = "SELENOCYSTEINE";
    m_DefaultStrCodes_map [SERINE] = "SERINE";
    m_DefaultStrCodes_map [THREONINE] = "THREONINE";
    m_DefaultStrCodes_map [TRYPTOPHANE] = "TRYPTOPHANE";
    m_DefaultStrCodes_map [TYROSINE] = "TYROSINE";
    m_DefaultStrCodes_map [VALINE] = "VALINE";
    m_DefaultStrCodes_map [ANY] = "ANY";
    m_DefaultStrCodes_map [TRANSLATION_STOP] = "TRANSLATION_STOP";
    m_DefaultStrCodes_map [GAP] = "GAP";
    m_DefaultStrCodes_map [UNKNOWN] = "UNKNOWN";
};
