/***************************************************************************
                          _aacodes.h  -  description
                             -------------------
    begin                : Tue Jul 8 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _AACODES_H
#define _AACODES_H

#include <string>
#include <map>
#include "_aminoacids.h"
#include "s_are.h"

namespace N_AminoAcid
{

using namespace std;

typedef E_AminoAcid aa_t;
typedef char aa1l_t;
typedef char aa3l_t [4];
typedef string aastr_t;

struct S_AACodes
{
    typedef map <char, E_AminoAcid> known_1l_codes_t;
    known_1l_codes_t m_Known1lCodes_map;

    typedef map <string, E_AminoAcid> known_3l_codes_t;
    known_3l_codes_t m_Known3lCodes_map;

    typedef map <string, E_AminoAcid> known_str_codes_t;
    known_str_codes_t m_KnownStrCodes_map;

    typedef map <E_AminoAcid, char> default_1l_codes_t;
    default_1l_codes_t m_Default1lCodes_map;

    typedef map <E_AminoAcid, string> default_3l_codes_t;
    default_3l_codes_t m_Default3lCodes_map;

    typedef map <E_AminoAcid, string> default_str_codes_t;
    default_str_codes_t m_DefaultStrCodes_map;

    S_AACodes ();
};

extern S_AACodes g_AACodes;

struct S_IsAACodeKnown : public unary_function <string, bool>
{
    bool operator () (const string & str) const
    {
        return g_AACodes.m_Known3lCodes_map.count (str);
    };

    bool operator () (char c) const
    {
        return g_AACodes.m_Known1lCodes_map.count (c);
    };
};
const S_IsAACodeKnown isAACodeKnown = S_IsAACodeKnown ();
const S_Are <S_IsAACodeKnown> areAACodesKnown = S_Are <S_IsAACodeKnown> ();


inline void convertAA (const string & AA_str, E_AminoAcid & AA)
{
    S_AACodes::known_3l_codes_t::const_iterator i;
    AA = ((i = g_AACodes.m_Known3lCodes_map.find (AA_str)) == g_AACodes.m_Known3lCodes_map.end ()) ?
         UNKNOWN : i->second;
};
inline void convertAA (const char AA_c, E_AminoAcid & AA)
{
    S_AACodes::known_1l_codes_t::const_iterator i;
    AA = ((i = g_AACodes.m_Known1lCodes_map.find (AA_c)) == g_AACodes.m_Known1lCodes_map.end ()) ?
         UNKNOWN : i->second;
};


inline void convertAA (const E_AminoAcid AA, char & c)
{
    c = g_AACodes.m_Default1lCodes_map [AA];
};
inline void convertAA (const E_AminoAcid AA, string & str)
{
    str = g_AACodes.m_Default3lCodes_map [AA];
};


inline string convertAA2Str (const E_AminoAcid AA)
{
    return g_AACodes.m_DefaultStrCodes_map [AA];
};
inline E_AminoAcid convertStr2AA (const string & str)
{
    S_AACodes::known_str_codes_t::const_iterator i;
    return ((i = g_AACodes.m_KnownStrCodes_map.find (str)) == g_AACodes.m_KnownStrCodes_map.end ()) ?
           UNKNOWN : i->second;
};
inline E_AminoAcid convertStr2AA (const string & str, bool Failbit)
{
    S_AACodes::known_str_codes_t::const_iterator i;
    if ((i = g_AACodes.m_KnownStrCodes_map.find (str)) == g_AACodes.m_KnownStrCodes_map.end ())
    {
        Failbit = true;
        return UNKNOWN;
    }
    else
        return i->second;
};


template <class T, class U>
inline T
convertAA (const U & u)
{
    T t;
    convertAA (u, t);
    return t;
};

template <class T, class U>
inline T
convertAA (const U & u, bool & Failbit)
{
    Failbit = ! isAACodeKnown (u);
    return convertAA <T> (u);
};

inline void convertAA (char AA_c, char & c)
{
    convertAA (convertAA <E_AminoAcid> (AA_c), c);
};

inline void convertAA (char AA_c, const string & str)
{
    convertAA (convertAA <E_AminoAcid> (AA_c), str);
};

inline void convertAA (const string & AA_str, const string & str)
{
    convertAA (convertAA <E_AminoAcid> (AA_str), str);
};

inline void convertAA (const string & AA_str, char & c)
{
    convertAA (convertAA <E_AminoAcid> (AA_str), c);
};


};

#endif
