/***************************************************************************
                          e_aminoacid.h  -  description
                             -------------------
    begin                : Thu Jun 19 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef E_AMINOACID_H
#define E_AMINOACID_H

#include <vector>
#include <set>
#include "s_are.h"

namespace N_AminoAcid
{
using namespace std;
using namespace N_Tools;


/** enumeration of the used amino acids
  *@author Cyril Deremble
  */
enum E_AminoAcid
{
    ALANINE = 0,
    ARGININE,
    ASPARAGINE,
    ASPARTATE,
    ASPARAGINE_OR_ASPARTATE,
    CYSTEINE,
    GLUTAMINE,
    GLUTAMATE,
    GLUTAMINE_OR_GLUTAMATE,
    GLYCINE,
    HISTIDINE,
    ISOLEUCINE,
    LEUCINE,
    LYSINE,
    METHIONINE,
    PHENYLALANINE,
    PROLINE,
    SELENOCYSTEINE,
    SERINE,
    THREONINE,
    TRYPTOPHANE,
    TYROSINE,
    VALINE,
    ANY,
    TRANSLATION_STOP,
    GAP,
    UNKNOWN
};

const size_t AMINO_ACID_COUNT = UNKNOWN + 1;

struct S_AAProperties
{
    set
        <E_AminoAcid> & nonAAs ()
    {
        return m_NonAAs_set;
    };
    set
        <E_AminoAcid> & hydrophobicAAs ()
    {
        return m_HydrophobicAAs_set;
    };
    set
        <E_AminoAcid> & clusterBreakerAAs ()
    {
        return m_ClusterBreakerAAs_set;
    };

    set
        <E_AminoAcid> nonAAs () const
        {
            return m_NonAAs_set;
        };
    set
        <E_AminoAcid> hydrophobicAAs () const
        {
            return m_HydrophobicAAs_set;
        };
    set
        <E_AminoAcid> clusterBreakerAAs () const
        {
            return m_ClusterBreakerAAs_set;
        };


    set
        <E_AminoAcid> m_NonAAs_set;
    set
        <E_AminoAcid> m_HydrophobicAAs_set;
    set
        <E_AminoAcid> m_ClusterBreakerAAs_set;

    S_AAProperties ();
};

extern S_AAProperties g_AAProperties;


struct S_IsAA : public unary_function <E_AminoAcid, bool>
{
    bool operator () (const E_AminoAcid & AA) const
    {
        return ! g_AAProperties.m_NonAAs_set.count (AA);
    };
};
const S_IsAA isAA = S_IsAA ();
const S_Are <S_IsAA> areAA = S_Are <S_IsAA> ();

struct S_IsHydrophobic : public unary_function <E_AminoAcid, bool>
{
    bool operator () (const E_AminoAcid & AA) const
    {
        return g_AAProperties.m_HydrophobicAAs_set.count (AA);
    };
};
const S_IsHydrophobic isHydrophobic = S_IsHydrophobic ();
const S_Are <S_IsHydrophobic> areHydrophobic = S_Are <S_IsHydrophobic> ();

struct S_IsGap : public unary_function <E_AminoAcid, bool>
{
    bool operator () (const E_AminoAcid & AA) const
    {
        return AA == GAP;
    };
};
const S_IsGap isGap = S_IsGap ();
const S_Are <S_IsGap> areGap = S_Are <S_IsGap> ();

struct S_IsProline : public unary_function <E_AminoAcid, bool>
{
    bool operator () (const E_AminoAcid & AA) const
    {
        return AA == PROLINE;
    };
};
const S_IsProline isProline = S_IsProline ();
const S_Are <S_IsProline> areProline = S_Are <S_IsProline> ();


struct S_IsClusterBreaker : public unary_function <E_AminoAcid, bool>
{
    bool operator () (const E_AminoAcid & AA) const
    {
        return g_AAProperties.m_ClusterBreakerAAs_set.count (AA);
    };
};
const S_IsClusterBreaker isClusterBreaker = S_IsClusterBreaker ();


};

#endif

