/***************************************************************************
                          _inputparams.cpp  -  description
                             -------------------
    begin                : Thu Jun 19 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "_inputparams.h"


static void displayHelp ();


// *                                                                         *
// readArg
// *                                                                         *


static void readArg (int & i, char argc, char *argv [], string & str)
{
    if ((i + 1) == argc)
    {
        throw C_Exception ("expecting argument after '" + string (argv [i]) + "'");
    }
    else if (argv [i + 1][0] == '-')
    {
        throw C_Exception ("expecting argument after '" + string (argv [i]) + "'");
    }
    else
    {
        str = argv [++ i];
    }
};


/***************************************************************************
 *                                                                         *
 *   readInputParams
 *                                                                         *
 ***************************************************************************/

void readInputParams (char argc, char *argv [])
{
	string InputParamFileName_str;

	for (int i = 0; i < argc; i ++)
	{
		string arg_i_str (argv [i]);

		if (arg_i_str.size () < 2)
			throw C_Exception (string ("invalid option ") + arg_i_str);
		else if (argv [i][0] != '-')
			g_InputFileNames_lst.push_back (arg_i_str);
		else if (arg_i_str == "-h" || arg_i_str == "--help")
		{
			displayHelp ();
			exit (1);
		}
		else if (arg_i_str == "-o-par")
		{
			if (! g_OutputParamFileName_str.empty ())
			{
				readArg (i, argc, argv, arg_i_str);
				throw C_Exception (string ("output parameter file name '") + arg_i_str +
						"' conflicts with '" + g_OutputParamFileName_str);
			}
			else
				readArg (i, argc, argv, g_OutputParamFileName_str);
		}
		else if (arg_i_str == "-o-mat")
		{
	  	        Align_b = false;
			if (! g_OutputDistMatFileName_str.empty ())
			{
				readArg (i, argc, argv, arg_i_str);
				throw C_Exception (string ("output distance matrix file name '") + arg_i_str +
						"' conflicts with '" + g_OutputDistMatFileName_str);
			}
			else
				readArg (i, argc, argv, g_OutputDistMatFileName_str);
		}
		else if (arg_i_str == "-i-tree")
		{
			if (! g_InputTreeFileName_str.empty ())
			{
				readArg (i, argc, argv, arg_i_str);
				throw C_Exception (string ("input tree file name '") + arg_i_str +
						"' conflicts with '" + g_InputTreeFileName_str);
			}
			else
				readArg (i, argc, argv, g_InputTreeFileName_str);
		}
		else if (arg_i_str [1] == 'o')
		{
			if (arg_i_str == "-o" || arg_i_str == "-o-xml")
				g_OutputFileFormat_e = XML;
			else if (arg_i_str == "-o-xml-3l")
				g_OutputFileFormat_e = XML_3L;
			else if (arg_i_str == "-o-fam")
				g_OutputFileFormat_e = FASTA;
			else if (arg_i_str == "-o-msf")
				g_OutputFileFormat_e = MSF;
			else if (arg_i_str == "-o-ffa")
				g_OutputFileFormat_e = FOLDFIT;
			else
				throw C_Exception (string ("unknown option ") + arg_i_str);
		
			if (! g_OutputFileName_str.empty ())
			{
				readArg (i, argc, argv, arg_i_str);
				throw C_Exception (string ("output file name '") + arg_i_str +
						"' conflicts with '" + InputParamFileName_str);
			}
			else
			{
				readArg (i, argc, argv, g_OutputFileName_str);
			}
		}
		else if (arg_i_str == "-i-nogap")
			ClearGaps_b = true;
		else if (arg_i_str == "-i-noblock")
			ClearClusters_b = true;
		else if (arg_i_str == "-i-nocolor")
			ClearColors_b = true;
		else if (arg_i_str == "-noblock")
			AnalyseClusters_b = false;
		else if (arg_i_str == "-nofillblocks")
			FillClusters_b = false;
		else if (arg_i_str == "-noalign")
			Align_b = false;
		else if (arg_i_str == "-i-noscoreconserved")
			ScoreConservedRes_b = false;
		else if (arg_i_str == "-noblockcolor")
			ColorClusters_b = false;
		else if (arg_i_str == "-noblockbreakercolor")
			ColorClusterBreakers_b = false;
		else if (arg_i_str == "-nocolor")
			ColorAAs_b = false;
		else if (arg_i_str == "-alignpairwise"){
		        AlignPairwiseToTarget_b = true;
		        Align_b = false;
		}
		else if (arg_i_str == "-constructblock")
			ConstructClustersFromIds_b = true;
		else if (arg_i_str == "-th-nonoverlap")
			clustAlign.m_Overlap = false;
		else if (arg_i_str == "-th-overlap")
			clustAlign.m_Overlap = true;
		else if (arg_i_str == "-globloc")
 		        GlobalLocalAlign_b = true;
		else if (arg_i_str == "-global")
		        GlobalLocalAlign_b = false;
		else if (arg_i_str == "-gappenalties"){
			i++;
			clustAlign.m_GOP = (score_t)atof(argv[i]);
			i++;
			clustAlign.m_GEP =  (score_t)atof(argv[i]);
			i++;
			clustAlign.m_ClustGOP =  (score_t)atof(argv[i]);
			i++;
			clustAlign.m_ClustGEP =  (score_t)atof(argv[i]);
		}
		else
		{
			throw C_Exception (string ("unknown option ") + arg_i_str);
		}
	}

	ifstream TestFile;
	for (list <C_FilePathName>::const_iterator i = g_InputFileNames_lst.begin ();
		    i != g_InputFileNames_lst.end (); i ++)
	{
		TestFile.open (i->c_str ());
		if (! TestFile.is_open ())
		{
			C_Exception Exception ("cannot open input file '");
			throw Exception << *i << "'";
		}
		TestFile.close ();
	}
};

// ***************************************************************************
// *                                                                         *
// *   displayHelp
// *                                                                         *
// ***************************************************************************

static void displayHelp ()
{
    cout << "\
    \
    Usage: phybal 'list of input file names' ... '-option' ['option arguments'] ... \n\
    \n\
    \n\
 ** INPUT: list of files containing sequences or parameters\n\
    supported format: XML, FASTA, MSF, FOLDFIT\n\
    \n\
    -i-nogap                  remove gaps in input sequences\n\
    -i-noblock                remove blocks in input sequences\n\
    -i-nocolor                clear amino acid colors in input sequences\n\
    \n\
    -i-tree   'filename'      specify an input file name conree (for future development)\n\
    \n\
    \n\
 ** OUTPUT:\n\
    \n\
    -o        'filename'      specify the output file name\n\
    -o-xml        \"           output format = XML\n\
    -o-xml-3l     \"           output format = XML with 3-letter amino acid code\n\
    -o-fam        \"           output format = fasta\n\
    -o-msf        \"           output format = MSF\n\
    -o-ffa        \"           output format = FoldFit Alignment\n\
    \n\
    -o-par        \"           specify the output file name for parameters\n\
    -o-mat        \"           specify the output file name for the distance matrix (for future development)\n\
    \n\
    \n\
 ** ANALYSIS:\n\
    \n\
    -noblock                  do not analyse blocks\n\
    -nofillblocks             do not fill block (scattering method)\n\
    -noalign                  do not align sequences\n\
    -noscoreconserved         do score conserved residues\n\
    -noblockcolor             do not color blocks\n\
    -nocolor                  do not color amino acids\n\
    -noblockbreakercolor      do not color block breakers\n\
    -alignpairwise            align all sequences pairwise to the first sequence\n\
    -constructblock           construct blocks from input ids\n\
    \n\
    \n\
 ** ALIGNMENT PARAMETERS: substitution matrices, hydrophobic amino acids, number of neighbors defining a block\n\
    and other parameters can be changed or specified in a xml file given as first parameter in the command line.\n\
    The format of the xml file must be the same as for the xml output file given by the -o-par option.\n\
    \n\
    -th-overlap               specify to use the IHBM matrix for pairs of residues where at least one residue belong to a hb (DEFAULT)\n\
    -th-nonoverlap            specify to use the IHBM matrix for pairs of residues both belonging to a hb\n\
    \n\
    -global                   global alignment (DEFAULT)\n\
    -globloc                  global-local alignment\n\
    \n\
    -gappenalties 'gop' 'gep' 'bgop' 'bgep'        define gap penalties\n\
    \n\
    \n\
 ** HELP:\n\
    \n\
    -h                        display this help\n\
    --help                    display this help\n\
    Bug report: Cyril Deremble <deremble@ihes.fr> and Julie Baussand <baussand@infop6.jussieu>\n"

    << endl;
};

