/***************************************************************************
				n_xmltools.cpp  -  description
				-------------------
begin                : Mon May 12 2003
copyright            : (C) 2003 by Cyril Deremble
email                : deremble@ihes.fr
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

#include "_xmltools.h"


using namespace N_XML;

#include <iostream>

/***************************************************************************
*                                                                         *
*   XML Header
*                                                                         *
***************************************************************************/

ostream &
N_XML::
		operator << (ostream & os, const S_XMLHeader & Header)
{
	os << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
	if (! Header.XSLStylesheet_str.empty ())
		os << "<?xml-stylesheet type=\"text/xsl\" href=\""
				<< Header.XSLStylesheet_str << "\"?>" << endl;
	if (! Header.DOCTYPE_str.empty ())
		os << "<!DOCTYPE " << Header.DOCTYPE_str << " >" << endl;
	return os;
};

/***************************************************************************
*                                                                         *
*   XML Attribute
*                                                                         *
***************************************************************************/

void
C_XMLTag::
		readAttributesFrom (istream & is)
{
	while (is.good ())
	{
		is >> ws;
		if (is.peek () == '/' || is.peek () == '>')
			break;
		else
		{
			string Name_str;
			char EqualDelim, QuoteDelim;
			is >> word >> Name_str;
			if (attributes.find (Name_str) != attributes.end () || ! is.good ())
				is.setstate (ios::failbit);
			else
			{
				is >> EqualDelim >> QuoteDelim;
				if (EqualDelim != '=' || QuoteDelim != '\"')
					is.setstate (ios::failbit);
				else
					is >> getStrictLine (QuoteDelim) >> attributes [Name_str];
			}
		}
	}
}

string
C_XMLTag::
		getAttributeValue (const string & AttributeName_str) const
{
	attributes_t::const_iterator i = attributes.find (AttributeName_str);
	return ((i != attributes.end ()) ? i->second : "");
};

istream &
C_XMLTag::
		getAttributeStream (const string & AttributeName_str) const
{
	m_AttributeStreamBuffer.clear ();
	m_AttributeStreamBuffer.str ().clear ();

	attributes_t::const_iterator i = attributes.find (AttributeName_str);

	if (i != attributes.end ())
		m_AttributeStreamBuffer << i->second;

	return m_AttributeStreamBuffer;
}


/***************************************************************************
*                                                                         *
*   XML Tag
*                                                                         *
***************************************************************************/


istream &
N_XML::
		operator >> (istream & is, C_XMLTag & Tag)
{
	char c;
	Tag.clearAll ();
	is >> c;
	if (c != '<' || is.peek () == '>' || is.peek () == '/')
	{
		is.putback (c);
		is.setstate (ios::failbit);
	}
	else if (is.peek () == '!')
	{
		is.get ();
		if (is.peek () == '-')
		{
			is.get ();
			if (is.get () != '-')
				is.setstate (ios::failbit);
			else
				while (! is.fail ())
					if (is.get () == '-')
						if (is.get () == '-')
							if (is.get () == '>')
								break;
		}
		else
			is >> line ('>');
		return is >> Tag;
	}
	else if (is.peek () == '?')
	{
		is.get ();
		while (! is.fail ())
			if (is.get () == '?')
				if (is.get () == '>')
					break;
		return is >> Tag;
	}
	else
	{

		is >> word >> Tag.name;
		Tag.readAttributesFrom (is);
		is >> c;
		if (c == '/')
		{
			if (is.get () != '>')
				is.setstate (ios::failbit);
		}
		else if (c != '>')
		{
			is.setstate (ios::failbit);
		}
		else
			while (is.good ())
		{
			if (is.peek () == '<')
			{
				is.get ();
				if (is.peek () == '/')
				{
					is.get ();
					string Buffer_str;
					is >> word >> Buffer_str >> c;
					if (Buffer_str != Tag.name)
						is.setstate (ios::failbit);
					break;
				}
				else if (is.peek () == '!')
				{
					is.get ();
					if (is.get () != '-')
						is.setstate (ios::failbit);
					else if (is.get () != '-')
						is.setstate (ios::failbit);
					else
						while (! is.fail ())
							if (is.get () == '-')
								if (is.get () == '-')
									if (is.get () == '>')
										break;
				}
				else if (is.peek () == '?')
				{
					is.get ();
					while (! is.fail ())
						if (is.get () == '?')
							if (is.get () == '>')
								break;
				}
				else
				{
					is.putback ('<');
					C_XMLTag TempTag;
					is >> TempTag;
					if (! is.fail ())
						Tag << TempTag;
				}
			}
			else
			{
				c = is.get ();
				if (! is.fail ())
					Tag << c;
			}
		}
	}
	return is;
};


ostream &
N_XML::
		operator << (ostream & os, const C_XMLTag & Tag)
{
	os << xml_begt (Tag.name, !Tag.attributes.empty ());
	for (C_XMLTag::att_const_iterator i = Tag.attributes.begin (); i != Tag.attributes.end (); i ++)
		os << xml_at (i->first, i->second);
	if (!Tag.attributes.empty ())
		os << xml_endats;

	os << Tag.str () << xml_endt (Tag.name);
	return os;
};

istream &
C_GetXMLTag::
		operator >> (C_XMLTag & Tag) const
{
	s ().seekg (0, m_Dir);
	do
	{
		C_XMLTag Temp;
		s () >> Temp;
		
		if (s ().fail ()) break;
		
		if (Temp.name == m_Name_str)
		{
			Tag = Temp;
			break;
		}
		
		if (m_IsModeRecursive_b)
		{
			istringstream Temp2 (Temp.str ());
			Temp2 >> xml_gettag (m_Name_str, true) >> Tag;
			break;
		}
	}
	while (! s ().fail ());
	return s ();
};

