/***************************************************************************
				_xmltools.h  -  description
				-------------------
begin                : Mon May 12 2003
copyright            : (C) 2003 by Cyril Deremble
email                : deremble@ihes.fr
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

#ifndef _XMLTOOLS_H
#define _XMLTOOLS_H

#include <ostream>
#include <map>
#include <sstream>
#include <iomanip>
#include "iomanip_ext.h"

namespace N_XML
{
	using namespace std;
	using namespace N_Tools;

	typedef unsigned long color_t;

	const color_t MAX_COLOR = 0xffffff;
	const color_t BLACK     = 0x000000;
	const color_t WHITE     = 0xffffff;
	const color_t RED       = 0xff0000;
	const color_t GREEN     = 0x00ff00;
	const color_t BLUE      = 0x0000ff;
	const color_t YELLOW    = 0xffff00;
	const color_t PINK      = 0xff00ff;
	const color_t CYAN      = 0x00ffff;
	const color_t ORANGE    = 0xffaa00;

	class C_HTMLColorIOManip : public C_IOStreamManip <istream, ostream>
	{
		public:
			template <class T>
					istream & operator >> (T & t) const
			{
				char c;
				is () >> c;
				if (c != s_ColorPrefix_c || isWhiteSpace (is ().peek ()))
					is ().setstate (ios::failbit);
				return is () >> hex >> t >> dec;
			};
			template <class T>
					ostream & operator << (const T & t) const
			{
				return os () << s_ColorPrefix_c <<
						setw (s_Width_n) << setfill (s_Fill_c) << hex << t << dec;
			};

		private:
			static const size_t s_Width_n = 6;
			static const char s_Fill_c = '0';
			static const char s_ColorPrefix_c = '#';
	};
	const C_HTMLColorIOManip htmlcolor = C_HTMLColorIOManip ();

	inline const C_HTMLColorIOManip & operator >> (istream & Stream, const C_HTMLColorIOManip & Manip)
	{
		Manip.setIStream (Stream);
		return Manip;
	};
	inline const C_HTMLColorIOManip & operator << (ostream & Stream, const C_HTMLColorIOManip & Manip)
	{
		Manip.setOStream (Stream);
		return Manip;
	};

	struct S_XMLHeader
	{
		string DOCTYPE_str;
		string XSLStylesheet_str;
	};

	ostream & operator << (ostream & os, const S_XMLHeader & Header);

	class C_XMLTag : public stringstream
	{
		public:
			typedef map <string, string> attributes_t;
			typedef attributes_t::iterator att_iterator;
			typedef attributes_t::const_iterator att_const_iterator;

			C_XMLTag (const string & Name_str = "", const string & Value_str = "")
			: stringstream (Value_str), name (Name_str)
			{}

			C_XMLTag & operator = (const C_XMLTag & Tag)
			{
				name = Tag.name;
				attributes = Tag.attributes;
				(*this) << Tag.str ();
				return *this;
			}

			string name;
			attributes_t attributes;

			string getAttributeValue (const string & AttributeName_str) const;
			istream & getAttributeStream (const string & AttributeName_str) const;
			istream & attributeStream () const
			{
				return m_AttributeStreamBuffer;
			};


			void clearAll ()
			{
				name.clear ();
				attributes.clear ();
				clear ();
			};

			void readAttributesFrom (istream & is);
		private:
			mutable stringstream m_AttributeStreamBuffer;
	};

	istream & operator >> (istream & is, C_XMLTag & Tag);
	ostream & operator << (ostream & os, const C_XMLTag & Tag);

	class C_GetXMLTag : public C_StreamManip <istream>
	{
		public:
			typedef ios::seekdir seekdir;
			C_GetXMLTag (const string & Name_str = "", bool IsModeRecursive_b = false, seekdir Dir = ios::cur)
			: m_Name_str (Name_str), m_IsModeRecursive_b (IsModeRecursive_b), m_Dir (Dir)
			{}
			;

			istream & operator >> (C_XMLTag & Tag) const;

			C_GetXMLTag operator () (const string & Name_str, bool IsModeRecursive_b = false, seekdir Dir = ios::cur) const
			{
				return C_GetXMLTag (Name_str, IsModeRecursive_b, Dir);
			};

		private:
			string m_Name_str;
			bool m_IsModeRecursive_b;
			seekdir m_Dir;
	};

	const C_GetXMLTag xml_gettag = C_GetXMLTag ();

	inline const C_GetXMLTag & operator >> (istream & Stream, const C_GetXMLTag & Manip)
	{
		Manip.setStream (Stream);
		return Manip;
	};

	inline string xml_begt (const string & Name_str, bool WithAttributes = false)
	{
		return ((WithAttributes) ? (string ("<") + Name_str) : (string ("<") + Name_str + '>'));
	};
	inline string xml_begendt (const string & Name_str)
	{
		return string ("<") + Name_str + "/>";
	};

	const bool xml_withat = true;

	template <class T>
			inline string xml_at (const string & Name_str, const T & Value)
	{
		ostringstream Value_str;
		Value_str << Value;
		return string (" ") + Name_str + "=\"" + Value_str.str () + '\"';
	};

	inline string xml_begat (const string & Name_str)
	{
		return string (" ") + Name_str + "=\"";
	};

	const char xml_endat = '\"';
	const char xml_endats = '>';
	const char xml_endtats [3] = "/>";

	inline string xml_endt (const string & Name_str)
	{
		return string ("</") + Name_str + '>';
	};

};

#endif
