/***************************************************************************
                          c_aacodesiomanip.h  -  description
                             -------------------
    begin                : Fri Aug 1 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_AACODESIOMANIP_H
#define C_AACODESIOMANIP_H

#include "iomanip_aa.h"

using namespace N_AminoAcid;

/**
  *@author Cyril Deremble
  */

class C_AACodesIOManip : public C_IOStreamManip <C_XMLTag, ostream>
{
public:
    C_XMLTag & operator >> (S_AACodes & AACodes) const;
    ostream & operator << (const S_AACodes & AACodes) const;

private:
    template <class T>
    void readAACodes (C_XMLTag & Tag, T & AACodes_map) const
    {
        typedef typename T::key_type key_type;
        Tag >> ws;
        while (Tag.good ())
        {
            C_XMLTag PairTag;
            Tag >> PairTag >> ws;
            if (PairTag.name != "pair")
                Tag.setstate (ios::failbit);
            else
            {
                E_AminoAcid AA;
                key_type Code = key_type ();
                PairTag.getAttributeStream ("aa-name") >> aastr >> AA;
                if (PairTag.attributeStream ().fail ())
                    Tag.setstate (ios::failbit);
                else
                {
                    PairTag.getAttributeStream ("code") >> Code;
                    if (PairTag.attributeStream ().fail ())
                        Tag.setstate (ios::failbit);
                    else
                    {
                        AACodes_map.insert (make_pair (Code, AA));
                    }
                }
            }
        }
    };

    template <class T>
    void writeAACodes (const T & AACodes_map) const
    {
        typedef typename T::const_iterator iterator;
        for (iterator i = AACodes_map.begin ();
                i != AACodes_map.end (); i ++)
            os () << xml_begt ("pair", xml_withat)
            << xml_at ("aa-name", convertAA2Str (i->first))
            << xml_at ("code", i->second) << xml_endtats << endl;
    };
};

const C_AACodesIOManip xml_aacodes = C_AACodesIOManip ();

inline const C_AACodesIOManip & operator >> (C_XMLTag & Stream, const C_AACodesIOManip & Manip)
{
    Manip.setIStream (Stream);
    return Manip;
};

inline const C_AACodesIOManip & operator << (ostream & Stream, const C_AACodesIOManip & Manip)
{
    Manip.setOStream (Stream);
    return Manip;
};


#endif
