/***************************************************************************
                          c_aasubsmatrixiomanip.h  -  description
                             -------------------
    begin                : Tue Jul 8 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_AASUBSMATRIXIOMANIP_H
#define C_AASUBSMATRIXIOMANIP_H

#include "iomanip_aa.h"
#include <iomanip>
#include "c_exception.h"

using namespace N_AminoAcid;

/**
  *@author Cyril Deremble
  */

class C_AASubsMatrixTextIOManip : public C_IOStreamManip <istream, ostream>
{
public:
    template <class matrix_t>
    istream &
    operator >> (matrix_t & Matrix) const
    {
        char c;
        E_AminoAcid AA;
        vector <E_AminoAcid> J_vct;
        typedef vector <E_AminoAcid>::const_iterator j_iterator;

        is () >> c;
        while (c == '#')
            is () >> line >> c;
        is ().putback (c);

        while (! isWhiteSpace (is ().peek ()))
        {
            J_vct.push_back (UNKNOWN);
            is () >> aa1l >> J_vct.back () >> horws;
        }

        while (is ().good ())
        {
            is () >> aa1l >> AA;
            for (j_iterator j = J_vct.begin (); j != J_vct.end () && ! is ().fail (); j ++)
                is () >> Matrix [AA][*j];
            is () >> ws;
        }
        return is ();
    };

    template <class matrix_t>
    ostream &
    operator << (const matrix_t & Matrix) const
    {
        char LastFill_c = os ().fill (' ');
        int LastWidth = os ().width (3);
        os () << ' ';
        for (size_t i = 0; i < AMINO_ACID_COUNT; i ++)
            os () << aa1l << E_AminoAcid (i);

        os () << '\n';
        for (size_t i = 0; i < AMINO_ACID_COUNT; i ++)
        {
            os () << aa1l << E_AminoAcid (i);
            for (size_t j = 0; j < AMINO_ACID_COUNT; j ++)
            {
                os () << Matrix [i][j];
            }
            os () << '\n';
        }
        os ().fill (LastFill_c);
        os ().width (LastWidth);
        return os ();
    };
};


const C_AASubsMatrixTextIOManip aasubsmatrix = C_AASubsMatrixTextIOManip ();

inline const C_AASubsMatrixTextIOManip & operator >> (istream & Stream, const C_AASubsMatrixTextIOManip & Manip)
{
    Manip.setIStream (Stream);
    return Manip;
};

inline const C_AASubsMatrixTextIOManip & operator << (ostream & Stream, const C_AASubsMatrixTextIOManip & Manip)
{
    Manip.setOStream (Stream);
    return Manip;
};


class C_AASubsMatrixXMLIManip : public C_StreamManip <C_XMLTag>
{
public:
    template <class matrix_t>
    C_XMLTag &
    operator >> (matrix_t & Matrix) const
    {
        C_XMLTag RowTag;
        C_XMLTag ColTag;
        E_AminoAcid AA;
        vector <E_AminoAcid> J_vct;
        char c;

        typedef vector <E_AminoAcid>::const_iterator j_iterator;

        try
        {
            if (s ().name != "score-matrix")
                throw C_Exception ("bad tag");
            s () >> c;
            s ().putback (c);
            if (c != '<')
                s () >> aasubsmatrix >> Matrix;
            else
            {
                string Code_str = s ().getAttributeValue ("aa-code");
                if (Code_str != "")
                    m_xml_iaa.setDefaultAAFormat (Code_str);
                else
                    m_xml_iaa.setDefaultAAFormat (aa1l.aaCodeStr ());

                readRowTagFrom (s (), RowTag);
                readColTagFrom (RowTag, ColTag);
                while (RowTag.good ())
                {
                    readColTagFrom (RowTag, ColTag);
                    readAAFrom (ColTag, AA);
                    J_vct.push_back (AA);
                }

                while (s ().good ())
                {
                    readRowTagFrom (s (), RowTag);
                    readColTagFrom (RowTag, ColTag);
                    readAAFrom (ColTag, AA);

                    for (j_iterator j = J_vct.begin (); j != J_vct.end (); j ++)
                    {
                        readColTagFrom (RowTag, ColTag);
                        ColTag >> Matrix [AA][*j];
                        if (ColTag.fail ())
                            throw C_Exception ("bad matrix value");
                    }
                }
            }
        }
        catch (C_Exception & e)
        {
            s ().setstate (ios::failbit);
        }
        return s ();
    };

private:
    void readRowTagFrom (istream & is, C_XMLTag & Tag) const
    {
        is >> Tag >> ws;
        if (is.fail () || Tag.name != "tr")
            throw C_Exception ("bad tag");
    };
    void readColTagFrom (istream & is, C_XMLTag & Tag) const
    {
        is >> Tag >> ws;
        if (is.fail () || Tag.name != "td")
            throw C_Exception ("bad tag");
    };
    void readAAFrom (istream & is, E_AminoAcid & AA) const
    {
        is >> m_xml_iaa >> AA;
        if (is.fail ())
            throw C_Exception ("bad amino acid code");
    };


    mutable C_XMLAAIFormat m_xml_iaa;
};

const C_AASubsMatrixXMLIManip xml_iaasubsmatrix = C_AASubsMatrixXMLIManip ();

inline const C_AASubsMatrixXMLIManip & operator >> (C_XMLTag & Stream, const C_AASubsMatrixXMLIManip & Manip)
{
    Manip.setStream (Stream);
    return Manip;
};


class C_AASubsMatrixXMLOManip : public C_StreamManip <ostream>
{
public:
    C_AASubsMatrixXMLOManip (bool IsInCluster_b = false)
            : m_IsInCluster_b (IsInCluster_b)
    {}
    ;

    template <class matrix_t>
    ostream & operator << (const matrix_t & Matrix) const
    {
        s () << xml_begt ("score-matrix", xml_withat) << xml_at ("aa-code", xml_oaa1l.aaCodeStr ());
        if (m_IsInCluster_b)
            s () << xml_at ("in-cluster", "true");
        s () << xml_endats << endl << xml_begt ("tr") << "       " << xml_begendt ("td") << "            ";

        for (size_t i = 0; i < AMINO_ACID_COUNT; i ++)
            s () << xml_begt ("td") << xml_oaa1l << E_AminoAcid (i) << xml_endt("td");
        s () << xml_endt("tr") << endl;

        for (size_t i = 0; i < AMINO_ACID_COUNT; i ++)
        {
            s () << xml_begt ("tr") << xml_begt ("td") << xml_oaa1l << E_AminoAcid (i) << xml_endt("td");
            for (size_t j = 0; j < AMINO_ACID_COUNT; j ++)
            {
                s () << "        " << xml_begt ("td") << setw (2) << Matrix [i][j] << xml_endt("td");
            }
            s () << xml_endt("tr") << endl;
        }
        return s () << xml_endt("score-matrix") << endl;
    };

    C_AASubsMatrixXMLOManip operator () (bool IsInCluster_b) const
    {
        return C_AASubsMatrixXMLOManip (IsInCluster_b);
    };

private:
    bool m_IsInCluster_b;
};

const C_AASubsMatrixXMLOManip xml_oaasubsmatrix = C_AASubsMatrixXMLOManip (false);

inline const C_AASubsMatrixXMLOManip & operator << (ostream & Stream, const C_AASubsMatrixXMLOManip & Manip)
{
    Manip.setStream (Stream);
    return Manip;
};





#endif
