/***************************************************************************
                          c_alignedseqsgroup.h  -  description
                             -------------------
    begin                : Mon Aug 4 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_ALIGNEDSEQSGROUP_H
#define C_ALIGNEDSEQSGROUP_H

#include "c_referencevector.h"

template <class base_iterator_t>
class C_GroupIterator : public vector <base_iterator_t>
{
public:
    typedef vector <base_iterator_t> base_class_t;
    typedef typename base_class_t::iterator iterator;
    typedef typename base_class_t::const_iterator const_iterator;

    C_GroupIterator & operator ++ ()
    {
        for (iterator i = base_class_t::begin (); i != base_class_t::end (); i ++)
            ++ (*i);
        return (*this);
    };
    C_GroupIterator operator ++ (int)
    {
        C_GroupIterator it (*this);
        for (iterator i = base_class_t::begin (); i != base_class_t::end (); i ++)
            ++ (*i);
        return it;
    };
    C_GroupIterator & operator += (int n)
    {
        for (iterator i = base_class_t::begin (); i != base_class_t::end (); i ++)
            (*i) += n;
        return (*this);
    };
    C_GroupIterator operator + (int n)
    {
        C_GroupIterator it (*this);
        for (iterator i = it.begin (); i != it.end (); i ++)
            (*i) += n;
        return it;
    };
    C_GroupIterator & operator -- ()
    {
        for (iterator i = base_class_t::begin (); i != base_class_t::end (); i ++)
            -- (*i);
        return (*this);
    };
    C_GroupIterator operator -- (int)
    {
        C_GroupIterator it (*this);
        for (iterator i = base_class_t::begin (); i != base_class_t::end (); i ++)
            -- (*i);
        return it;
    };
    C_GroupIterator & operator -= (int n)
    {
        for (iterator i = base_class_t::begin (); i != base_class_t::end (); i ++)
            (*i) -= n;
        return (*this);
    };
    C_GroupIterator operator - (int n)
    {
        C_GroupIterator it (*this);
        for (iterator i = it.begin (); i != it.end (); i ++)
            (*i) -= n;
        return it;
    };
};


template <class value_t>
class C_AlignedSeqsGroup : public C_ReferenceVector <value_t>
{
public:
    typedef C_ReferenceVector <value_t> base_class_type;

    typedef typename value_t::value_type value_type;
    typedef typename value_t::iterator value_iterator;
    typedef typename value_t::const_iterator value_const_iterator;

    typedef C_GroupIterator <typename base_class_type::value_type::iterator> iterator;
    typedef C_GroupIterator <typename base_class_type::value_type::const_iterator> const_iterator;

    iterator begin ()
    {
        iterator it;
        for (typename base_class_type::iterator i = base_class_type::begin (); i != base_class_type::end (); i ++)
            it.push_back (i->begin ());
        return it;
    };
    const_iterator begin () const
    {
        const_iterator it;
        for (typename base_class_type::const_iterator i = base_class_type::begin (); i != base_class_type::end (); i ++)
            it.push_back (i->begin ());
        return it;
    };
    iterator end ()
    {
        iterator it;
        for (typename base_class_type::iterator i = base_class_type::begin (); i != base_class_type::end (); i ++)
            it.push_back (i->end ());
        return it;
    };
    const_iterator end () const
    {
        const_iterator it;
        for (typename base_class_type::const_iterator i = base_class_type::begin (); i != base_class_type::end (); i ++)
            it.push_back (i->end ());
        return it;
    };

    iterator insert (iterator it, const value_type & Value)
    {
        typename iterator::iterator i;
        typename base_class_type::iterator j;
        for (j = base_class_type::begin (), i = it.begin (); j != base_class_type::end () && i != it.end (); j ++, i ++)
            *i = j->insert (*i, Value);

        return it;
    };

    size_t size () const
    {
        return (base_class_type::empty ()) ? 0 : (base_class_type::begin ())->size ();
    };

};

#endif
