/***************************************************************************
				c_analysedseq.cpp  -  description
				-------------------
begin                : Mon May 5 2003
copyright            : (C) 2003 by Cyril Deremble
email                : deremble@ihes.fr
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

#include "c_analysedseq.h"



/***************************************************************************
*                                                                         *
*   clearClusters                                                         *
*                                                                         *
***************************************************************************/

void
C_AnalysedSeq::
clearClusters ()
{
	for (iterator i = begin (); i != end (); i ++)
	{
		i->Prev = CLUSTER_END;
		i->Next = CLUSTER_END;
		i->ClusterColor = WHITE;
		i->ClusterId = NO_ID;
	}
}




/***************************************************************************
*                                                                         *
*   constructClustersFromIds                                              *
*                                                                         *
***************************************************************************/

void
C_AnalysedSeq::
		constructClustersFromIds ()
{
	for (iterator i = begin (); i != end (); i ++)
	{
		i->Prev = CLUSTER_END;
		i->Next = CLUSTER_END;
	}
	
	map <int, cluster_t> Clusters_map;
	for (cluster_t::iterator i = cluster_t::iterator (begin (), begin (), end ()); i != end (); i ++)
		if (i->ClusterId != NO_ID)
		{
			map <int, cluster_t>::iterator Cluster = Clusters_map.find (i->ClusterId);
			if (Cluster == Clusters_map.end ())
			{
				Clusters_map.insert (make_pair (i->ClusterId, cluster_t (i, begin (), end ())));
			}
			else
			{
				Cluster->second.add (cluster_t::iterator (i, begin (), end ()));
			}
		}
}


/***************************************************************************
*                                                                         *
*   setClusterIds                                                         *
*                                                                         *
***************************************************************************/

void
C_AnalysedSeq::
		setClusterIds ()
{
	for (clust_iterator i = beginCluster (); i != end (); i ++)
	{
		C_AnalysedSeq::cluster_t Cluster_i = *i;
		Cluster_i.setId ();
	}
}


/***************************************************************************
*                                                                         *
*   colorClusters                                                         *
*                                                                         *
***************************************************************************/

void
C_AnalysedSeq::
		colorClusters (const vector <color_t> & Colors_vct)
{
	vector <color_t>::const_iterator Color_i;
	clust_iterator i;
	for (i = beginCluster (), Color_i = Colors_vct.begin (); i != end (); i ++, Color_i ++)
	{
		if (Color_i == Colors_vct.end ())
			Color_i = Colors_vct.begin ();

		cluster_t Cluster_i = *i;
		Cluster_i.color (*Color_i);
	}
}


/***************************************************************************
*                                                                         *
*   fillClusters                                                          *
*                                                                         *
***************************************************************************/

void
C_AnalysedSeq::
		fillClusters ()
{
	for (clust_iterator i = beginCluster (); i != end (); i ++)
	{
		C_AnalysedSeq::cluster_t Cluster_i = *i;
		Cluster_i.fill ();
	}
}


/***************************************************************************
*                                                                         *
*   colorClusterBreakers                                                  *
*                                                                         *
***************************************************************************/

void
C_AnalysedSeq::
		colorClusterBreakers (color_t Color)
{
	for (iterator i = begin (); i != end (); i ++)
		if (isClusterBreaker (*i))
			i->Color = Color;
}


/***************************************************************************
*                                                                         *
*   remove_if                                                             *
*                                                                         *
***************************************************************************/

void
C_AnalysedSeq::
		remove_if (E_AminoAcid AA)
{
	iterator i = begin ();
	while (i != end ())
		if ((*i) == AA)
			i = erase (i);
	else
		i ++;
}


/***************************************************************************
*                                                                         *
*   replace_if                                                            *
*                                                                         *
***************************************************************************/

void
C_AnalysedSeq::
		replace_if (E_AminoAcid AA, E_AminoAcid ByAA)
{
	for (iterator i = begin (); i != end (); i ++)
		if ((*i) == AA)
			*i = ByAA;
}


/***************************************************************************
*                                                                         *
*   color_if 
*                                                                         *
***************************************************************************/

void
C_AnalysedSeq::
		color_if (E_AminoAcid AA, color_t ByColor)
{
	for (iterator i = begin (); i != end (); i ++)
		if ((*i) == AA)
			i->Color = ByColor;
}


/***************************************************************************
*                                                                         *
*   clearClusterColors
*                                                                         *
***************************************************************************/

void
C_AnalysedSeq::
		clearClusterColors ()
{
	for (clust_iterator i = beginCluster (); i != end (); i ++)
	{
		cluster_t Cluster_i = *i;
		Cluster_i.color (WHITE);
	}
}

/***************************************************************************
*                                                                         *
*   clearClusterColors
*                                                                         *
***************************************************************************/

void
C_AnalysedSeq::
		clearColors ()
{
	for (iterator i = begin (); i != end (); i ++)
		i->Color = BLACK;
}

/***************************************************************************
*                                                                         *
*   clearConservedScore
*                                                                         *
***************************************************************************/

void C_AnalysedSeq::clearConservedScore ()
{
	for (iterator i = begin (); i != end (); i ++)
		i->ConservedScore = 0;
}


template <> size_t C_AnalysedSeq::cluster_t::s_IdCount = NO_ID;



