/***************************************************************************
                          c_analysedseq.h  -  description
                             -------------------
    begin                : Mon May 5 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_ANALYSEDSEQ_H
#define C_ANALYSEDSEQ_H

#include "n_aminoacid.h"
#include "c_testiterator.h"
#include "c_clusteredaaiterator.h"
#include "c_cluster.h"
#include "c_clusteriterator.h"

/**
  *@author Cyril Deremble
  */

class C_SeqInfo
{
	public:
		C_SeqInfo (const string & Name_str = string (),
			   const string & Comments_str = string ())
		    : m_Name_str (Name_str), m_Comments_str (Comments_str)
		{}
		;
		
		const string & name () const
		{
		    return m_Name_str;
		};
		string & name ()
		{
			return m_Name_str;
		};

		const string & comments () const
		{
			return m_Comments_str;
		};
		string & comments ()
		{
			return m_Comments_str;
		};

	private:
		string m_Name_str;
		string m_Comments_str;
};



class C_AnalysedSeq : public vector <S_AnalysedAA>, public C_SeqInfo
{
	public:
		typedef vector <S_AnalysedAA> base_class_t;

		typedef C_ClusterIterator <value_type, iterator> clust_iterator;
		typedef clust_iterator::cluster_t cluster_t;

		typedef C_ClusterIterator <const value_type, const_iterator> clust_const_iterator;
		typedef clust_const_iterator::cluster_t const_cluster_t;

		
		explicit C_AnalysedSeq (const string & Sequence_str = string (),
					const string & Name_str = string (),
					const string & Comments_str = string ())
		    :   base_class_t (Sequence_str.begin (), Sequence_str.end ()),
		    C_SeqInfo (Name_str, Comments_str)
		    { }

		C_AnalysedSeq (const string & Sequence_str,
			       const C_SeqInfo & SeqInfo)
		    :   base_class_t (Sequence_str.begin (), Sequence_str.end ()),
		    C_SeqInfo (SeqInfo)
		    {}

		template <class T>
				C_AnalysedSeq (T Begin, T End, const C_SeqInfo & SeqInfo)
			: base_class_t (Begin, End), C_SeqInfo (SeqInfo)
		{}

		~C_AnalysedSeq ()
		{}
		
		using base_class_t::begin;
		using base_class_t::end;
		
		clust_const_iterator beginCluster () const
		{
			return clust_const_iterator (begin (), begin (), end ());
		}
		
		clust_iterator beginCluster ()
		{
			return clust_iterator (begin (), begin (), end ());
		}

		void constructClustersFromIds ();
		void clearClusters ();
		void setClusterIds ();
		void fillClusters ();

		void colorClusters (const vector <color_t> & Colors_vct);
		void clearClusterColors ();
		
		void clearColors ();
		
		void colorClusterBreakers (color_t Color);
		
		void clearConservedScore ();
		
		template <class unaryTest_t>
		C_TestIterator <unaryTest_t, value_type, iterator>
		begin (const unaryTest_t & Test = unaryTest_t ())
		{
			return C_TestIterator <unaryTest_t, value_type, iterator>
					(begin (), begin (), end (), Test);
		}

		template <class unaryTest_t>
		C_TestIterator <unaryTest_t, const value_type, const_iterator>
		begin (const unaryTest_t & Test = unaryTest_t ()) const
		{
			return C_TestIterator <unaryTest_t, const value_type, const_iterator>
					(begin (), begin (), end (), Test);
		}

		template <class unaryTest_t>
		void color_if (color_t ByColor, const unaryTest_t & UnaryTest = unaryTest_t ());

		void color_if (E_AminoAcid AA, color_t ByColor);

		template <class unaryTest_t>
		void remove_if (const unaryTest_t & UnaryTest = unaryTest_t ());
		void remove_if (E_AminoAcid AA);

		template <class unaryTest_t>
		void replace_if (E_AminoAcid ByAA,
					const unaryTest_t & UnaryTest = unaryTest_t ());
		void replace_if (E_AminoAcid Val, E_AminoAcid ByAA);
};

/***************************************************************************
*                                                                         *
*   color_if                                                              *
*                                                                         *
***************************************************************************/

template <class unaryTest_t> void
C_AnalysedSeq::
		color_if (color_t ByColor, const unaryTest_t & Test)
{
	for (C_TestIterator <unaryTest_t, value_type, iterator>
		    i = begin <unaryTest_t> (Test);
		    i != end (); i ++)
		i->Color = ByColor;
};


/***************************************************************************
*                                                                         *
*   remove_if                                                             *
*                                                                         *
***************************************************************************/

template <class unaryTest_t> void
C_AnalysedSeq::
		remove_if (const unaryTest_t & Test)
{
	iterator i = begin ();
	while (i != end ())
		if (Test (*i))
			i = erase (i);
	else
		i ++;
};


/***************************************************************************
*                                                                         *
*   replace_if                                                            *
*                                                                         *
***************************************************************************/

template <class unaryTest_t> void
C_AnalysedSeq::
		replace_if (E_AminoAcid ByAA, const unaryTest_t & Test)
{
	for (C_TestIterator <unaryTest_t, value_type, const_iterator>
		    i = begin <unaryTest_t> (Test);
		    i != end (); i ++)
		i->Value = ByAA;
};

#endif
