/***************************************************************************
                          c_analysedseqs.cpp  -  description
                             -------------------
    begin                : Thu Jul 24 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "c_analysedseqs.h"
#include <iostream>
align_score_t C_AnalysedSeqs::align ()
{
	m_AlignmentScore = 0;
	C_AlignedSeqsGroup <value_type> AlignedSeqs;
	for (iterator i = begin (), j = ++ begin (); j != end (); i ++, j ++)
	{
		AlignedSeqs.push_back (*i);
		
		m_AlignmentScore += clustAlign (AlignedSeqs, *j);
	}
	areAligned () = true;

	m_AlignmentScore /= sequencesSize ();
	
	return m_AlignmentScore;
}


align_score_t C_AnalysedSeqs::align (const C_TreeNode <string> & Tree)
{
	typedef C_TreeNode <string> tree_type;
	typedef C_AlignedSeqsGroup <value_type> alignedseqs_t;
	
	m_AlignmentScore = alignNode (Tree);
	
	m_AlignmentScore /= sequencesSize ();
	
	areAligned () = true;
	
	return m_AlignmentScore;
}


align_score_t C_AnalysedSeqs::alignNode (const C_TreeNode <string> & Node)
{
	typedef C_TreeNode <string> tree_type;
	typedef C_AlignedSeqsGroup <value_type> alignedseqs_t;
	
	align_score_t AlignmentScore = 0;
	
	alignedseqs_t AlignedSeqs;
	for (tree_type::childs_type::const_iterator i = Node.childs ().begin (); i != Node.childs ().end (); i ++)
	{
		alignedseqs_t NextAlignedSeqs;
	
		if (i->isLeaf ())
		{
			NextAlignedSeqs.push_back ((*this) [i->value ()]);
		}
		else
		{
			AlignmentScore += alignNode (*i);
			for (tree_type::leaves_value_const_iterator j = i->beginLeavesValue (); j != i->endLeavesValue (); ++ j)
			{
				NextAlignedSeqs.push_back ((*this) [*j]);
			}
		}
		
		if (AlignedSeqs.empty ())
		{
			AlignedSeqs = NextAlignedSeqs;
		}
		else
		{
			if (! NextAlignedSeqs.empty ())
			{
				AlignmentScore += clustAlign (AlignedSeqs, NextAlignedSeqs);
		
				for (alignedseqs_t::base_class_type::iterator j = NextAlignedSeqs.alignedseqs_t::base_class_type::begin (); j != NextAlignedSeqs.alignedseqs_t::base_class_type::end (); ++ j)
				{
					AlignedSeqs.push_back ((*this) [j->name ()]);
				}
			}
		}
	}
	
	return AlignmentScore;
}




/************************/ 
/* iterative refinement */
/************************/


/*
align_score_t C_AnalysedSeqs::alignIter (const C_TreeNode <string> & Tree)
{
	typedef C_TreeNode <string> tree_type;
	typedef C_AlignedSeqsGroup <value_type> alignedseqs_t;
	
	m_AlignmentScore = alignNodeIter (Tree);
	m_AlignmentScore /= sequencesSize ();
	
	areAligned () = true;
	
	return m_AlignmentScore;
}


align_score_t C_AnalysedSeqs::alignNodeIter (const C_TreeNode <string> & Node)
{
	typedef C_TreeNode <string> tree_type;
	typedef C_AlignedSeqsGroup <value_type> alignedseqs_t;
	
	align_score_t AlignmentScore = 0;
	
	alignedseqs_t AlignedSeqs;
	for (tree_type::childs_type::const_iterator i = Node.childs ().begin (); i != Node.childs ().end (); i ++)
	{
		alignedseqs_t NextAlignedSeqs;
	
		if (i->isLeaf ())
		{
			NextAlignedSeqs.push_back ((*this) [i->value ()]);
		}
		else
		{
			AlignmentScore += alignNode (*i);
			for (tree_type::leaves_value_const_iterator j = i->beginLeavesValue (); j != i->endLeavesValue (); ++ j)
			{
				NextAlignedSeqs.push_back ((*this) [*j]);
			}
		}
		
		if (AlignedSeqs.empty ())
		{
			AlignedSeqs = NextAlignedSeqs;
		}
		else
		{
			if (! NextAlignedSeqs.empty ())
			{
				AlignmentScore += clustAlign (AlignedSeqs, NextAlignedSeqs);
		
				for (alignedseqs_t::base_class_type::iterator j = NextAlignedSeqs.alignedseqs_t::base_class_type::begin (); j != NextAlignedSeqs.alignedseqs_t::base_class_type::end (); ++ j)
				{
					AlignedSeqs.push_back ((*this) [j->name ()]);
				}
			}
		}
	}
	
	return AlignmentScore;
}

*/
