/***************************************************************************
                          c_cluster.h  -  description
                             -------------------
    begin                : Mon May 5 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_CLUSTER_H
#define C_CLUSTER_H

#include "c_clusteredaaiterator.h"


/***************************************************************************
 *                                                                         *
 *   Class declaration : C_Cluster                                         *
 *                                                                         *
 ***************************************************************************/


/**
  *@author Cyril Deremble
  */

template <class value_t, class iterator_t>
class C_Cluster
{
public:
    typedef iterator_t seq_iterator;
    typedef value_t value_type;
    typedef C_ClusteredAAIterator <value_type, seq_iterator> iterator;

    C_Cluster ()
    {}
    ;
    C_Cluster (const seq_iterator & it, const seq_iterator & BeginSeq_it, const seq_iterator & EndSeq_it)
            : m_Begin_it (it, BeginSeq_it, EndSeq_it)
    {
        init ();
    };

    explicit C_Cluster (const iterator & it)
            : m_Begin_it (it)
    {
        init ();
    };

    /*    template <class T>
        explicit C_Cluster (const T & it)
            : m_Begin_it ((iterator) it) {init ();};*/

    template <class T, class U>
    explicit C_Cluster (const C_Cluster <T, U> & Cluster)
            :   m_Size_n (Cluster.size ()),
            m_Begin_it ((iterator) Cluster.begin ()),
            m_Last_it ((iterator) (-- Cluster.end ()))
    {}
    ;

    iterator begin () const
    {
        return m_Begin_it;
    };
    iterator end () const
    {
        return ++ (iterator (m_Last_it));
    };

    //  seq_iterator beginSeq () const {return m_BeginSeq_it;};
    //  seq_iterator endSeq () const {return m_EndSeq_it;};

    value_type back () const
    {
        return *m_Last_it;
    };

    size_t size () const
    {
        return m_Size_n;
    };

    bool empty () const
    {
        return m_Size_n == 0;
    };

    bool surrounds (const seq_iterator & AA_it) const
    {
        return (m_Begin_it <= AA_it || m_Last_it >= AA_it);
    };
    bool contains (const seq_iterator & AA_it) const;

    iterator add (const iterator & AA_it);
                
    iterator add_safe (const iterator & AA_it);
                
    iterator remove (const iterator & it);

    void clear ();

    void merge (C_Cluster & Cluster);

    void color (color_t Color)
    {
        for (iterator i = begin (); i != end (); i ++)
            i->ClusterColor = Color;
    };

    size_t id () const
    {
        return begin()->ClusterId;
    };
    void setId ()
    {
        s_IdCount ++;
        for (iterator i = begin (); i != end (); i ++)
            i->ClusterId = s_IdCount;
    };

    static void resetIdCount (size_t Id = NO_ID)
    {
        s_IdCount = Id;
    };
    
    void fill ();

private:
    iterator m_Begin_it;
    iterator m_Last_it;

    size_t m_Size_n;

    mutable size_t m_Id;
    static size_t s_IdCount;

    void init ();
};


/***************************************************************************
 *                                                                         *
 *   init                                                                  *
 *                                                                         *
 ***************************************************************************/

template <class value_t, class iterator_t>
void
C_Cluster <value_t, iterator_t>::
init ()
{
    m_Size_n = 0;

    m_Last_it = m_Begin_it;
    ++ m_Size_n;
    while (m_Begin_it->Prev)
    {
        m_Size_n ++;
        -- m_Begin_it;
    }
    while (m_Last_it->Next)
    {
        ++ m_Size_n;
        ++ m_Last_it;
    }
};


/***************************************************************************
 *                                                                         *
 *   add                                                                   *
 *                                                                         *
 ***************************************************************************/

template <class value_t, class iterator_t>
typename C_Cluster <value_t, iterator_t>::iterator
C_Cluster <value_t, iterator_t>::
add (const iterator & AA_it)
{
    if (m_Size_n == 0)
    {
        m_Begin_it = AA_it;
        m_Begin_it->Prev = CLUSTER_END;
        m_Begin_it->Next = CLUSTER_END;

        m_Last_it = m_Begin_it;

        m_Size_n ++;
    }
    else if (m_Begin_it > AA_it)
    {
        size_t dist = m_Begin_it - AA_it;
        m_Begin_it->Prev = dist;
        m_Begin_it = AA_it;
        m_Begin_it->Prev = CLUSTER_END;
        m_Begin_it->Next = dist;

        m_Size_n ++;
    }
    else if (m_Last_it < AA_it)
    {
        size_t dist = AA_it - m_Last_it;
        m_Last_it->Next = dist;
        m_Last_it = AA_it;
        m_Last_it->Prev = dist;
        m_Last_it->Next = CLUSTER_END;

        m_Size_n ++;
    }
    else
    {
        for (iterator i = begin (), j = ++ begin (); j != end (); i ++, j ++)
        {
            if (i < AA_it && j > AA_it)
            {
                i->Next = AA_it - i;
                AA_it->Prev = i->Next;
                AA_it->Next = j - AA_it;
                j->Prev = AA_it->Next;

                m_Size_n ++;

                break;
            }
        }
    }

    return AA_it;
};


/***************************************************************************
 *                                                                         *
 *   add_safe                                                              *
 *                                                                         *
 ***************************************************************************/

template <class value_t, class iterator_t>
typename C_Cluster <value_t, iterator_t>::iterator
C_Cluster <value_t, iterator_t>::
add_safe (const iterator & AA_it)
{
    if (! contains (AA_it))
    {
        if (isInCluster (*AA_it))
        {
            C_Cluster <value_t, iterator_t> Cluster (AA_it);
            merge (Cluster);
        }
        else
        {
            add (AA_it);
        }
    }

    return AA_it;
};


/***************************************************************************
 *                                                                         *
 *   remove                                                                *
 *                                                                         *
 ***************************************************************************/

template <class value_t, class iterator_t>
typename C_Cluster <value_t, iterator_t>::iterator
C_Cluster <value_t, iterator_t>::
remove
    (const iterator & it)
{
    iterator Prev_it = it;
    iterator Next_it = it;

    if (it == begin () && it == m_Last_it)
    {
        -- m_Last_it;
        Next_it = end ();
    }
    else
    {
        if (it == begin ())
        {
            ++ m_Begin_it;
            m_Begin_it->Prev = CLUSTER_END;
            ++ Next_it;
        }
        else
        {
            if (it == m_Last_it)
            {
                -- m_Last_it;
                m_Last_it->Next = CLUSTER_END;
                Next_it = end ();
            }
            else
            {
                -- Prev_it;
                Prev_it->Next = Prev_it->Next + it->Next;

                ++ Next_it;
                Next_it->Prev = it->Prev + Next_it->Prev;
            }
        }
    }

    it->Next = CLUSTER_END;
    it->Prev = CLUSTER_END;
    it->ClusterColor = WHITE;
    it->ClusterId = 0;

    m_Size_n --;

    return Next_it;
};


/***************************************************************************
 *                                                                         *
 *   clear                                                                 *
 *                                                                         *
 ***************************************************************************/

template <class value_t, class iterator_t>
void
C_Cluster <value_t, iterator_t>::
clear ()
{
    iterator i = begin ();
    while (i != end ())
    {
        iterator j = i ++;
        j->Prev = CLUSTER_END;
        j->Next = CLUSTER_END;
        i->ClusterColor = WHITE;
        i->ClusterId = 0;
    }
};


/***************************************************************************
 *                                                                         *
 *   merge                                                                 *
 *                                                                         *
 ***************************************************************************/

template <class value_t, class iterator_t>
void
C_Cluster <value_t, iterator_t>::
merge (C_Cluster & Cluster)
{
    iterator i = Cluster.begin ();
    iterator End = Cluster.end ();
    while (i != End)
    {
        add (i ++);
    }
    Cluster = (*this);
};


/***************************************************************************
 *                                                                         *
 *   contains                                                              *
 *                                                                         *
 ***************************************************************************/

template <class value_t, class iterator_t>
bool
C_Cluster <value_t, iterator_t>::
contains (const seq_iterator & AA_it) const
{
    for (iterator i = begin (); i != end (); i ++)
        if (i == AA_it)
            return true;

    return false;
};


/***************************************************************************
 *                                                                         *
 *   fill                                                                  *
 *                                                                         *
 ***************************************************************************/

template <class value_t, class iterator_t>
void
C_Cluster <value_t, iterator_t>::
fill ()
{
    for (seq_iterator i = begin (); m_Last_it != i; ++ i)
    {
	    add_safe (iterator (i, begin ().begin (), begin ().end ()));
    }
};


#endif
