/***************************************************************************
                          c_displayparameters.cpp  -  description
                             -------------------
    begin                : Fri Aug 1 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "c_displayparameters.h"


C_DisplayParameters::
C_DisplayParameters ()
{
    m_ClusterBreakersColor = RED;

    m_ClusterColors_vct.push_back (BLUE);
    m_ClusterColors_vct.push_back (CYAN);
    m_ClusterColors_vct.push_back (GREEN);

    m_AAsColors_map [GLYCINE] = ORANGE;
    m_AAsColors_map [SERINE] = GREEN;
    m_AAsColors_map [THREONINE] = GREEN;

    m_OutputStylesheetHRef_str = "http://www.ihes.fr/GENOMIQUE/Hcaal/Sources/sequences_xml2html.xsl";

    m_SeqLineWidth_n = 100;
}

C_DisplayParameters g_DisplayParameters = C_DisplayParameters ();


/***************************************************************************
 *                                                                         *
 *   operator >>
 *                                                                         *
 ***************************************************************************/

C_XMLTag &
C_DisplayParametersIOManip::
operator >> (C_DisplayParameters & DisplayParams) const
{
    is () >> ws;
    if (is ().name != "display-parameters")
        is ().setstate (ios::failbit);
    else
        while (is ().good ())
        {
            C_XMLTag ParamTag;
            is () >> ParamTag >> ws;
            ParamTag >> ws;

            if (ParamTag.name == "cluster-colors")
            {
                DisplayParams.clusterColors ().clear ();
                while (ParamTag.good ())
                {
                    C_XMLTag ColorTag;
                    color_t Color;
                    ParamTag >> ColorTag >> ws;
                    if (ColorTag.name != "color")
                        ParamTag.setstate (ios::failbit);
                    else
                    {
                        ColorTag >> htmlcolor >> Color;
                        if (! ColorTag.fail ())
                            DisplayParams.clusterColors ().push_back (Color);
                        else
                            ParamTag.setstate (ios::failbit);
                    }
                }
            }
            else if (ParamTag.name == "cluster-breakers-color")
                ParamTag >> htmlcolor >> DisplayParams.clusterBreakersColor ();
            else if (ParamTag.name == "output-stylesheet")
                ParamTag >> DisplayParams.outputStylesheetHRef ();
            else if (ParamTag.name == "seq-line-width")
                ParamTag >> DisplayParams.seqLineWidth ();
            else if (ParamTag.name == "colored-aas")
            {
                DisplayParams.aasColors ().clear ();
                while (ParamTag.good ())
                {
                    S_AnalysedAA AA;
                    ParamTag >> xml_iaa >> AA >> ws;
                    if (! ParamTag.fail ())
                        DisplayParams.aasColors ().insert (make_pair (AA.Value, AA.Color));
                    else
                        ParamTag.setstate (ios::failbit);
                }
            }
            else
                ParamTag.setstate (ios::failbit);

            if (ParamTag.fail ())
                is ().setstate (ios::failbit);
        }
    return is ();
};


/***************************************************************************
 *                                                                         *
 *   operator <<
 *                                                                         *
 ***************************************************************************/

ostream &
C_DisplayParametersIOManip::
operator << (const C_DisplayParameters & DisplayParams) const
{
    os () << xml_begt ("display-parameters") << endl

    << xml_begt ("cluster-colors") << endl;
    C_DisplayParameters::cluster_colors_t ClusterColors = DisplayParams.clusterColors ();
    for (C_DisplayParameters::cluster_colors_t::const_iterator i = ClusterColors.begin ();
            i != ClusterColors.end (); i ++)
        os () << xml_begt ("color") << htmlcolor << *i << xml_endt ("color") << endl;
    os () << xml_endt ("cluster-colors") << endl

    << xml_begt ("cluster-breakers-color")
    << htmlcolor << DisplayParams.clusterBreakersColor ()
    << xml_endt ("cluster-breakers-color") << endl


    << xml_begt ("colored-aas") << endl;
    C_DisplayParameters::aas_colors_t AAsColors = DisplayParams.aasColors ();
    for (C_DisplayParameters::aas_colors_t::const_iterator i = AAsColors.begin ();
            i != AAsColors.end (); i ++)
        os () << xml_oaa1l << S_AnalysedAA (i->first, 0, 0, i->second) << endl;
    os () << xml_endt ("colored-aas") << endl

    << xml_begt ("output-stylesheet") << DisplayParams.outputStylesheetHRef ()
    << xml_endt ("output-stylesheet") << endl

    << xml_begt ("seq-line-width") << DisplayParams.seqLineWidth ()
    << xml_endt ("seq-line-width") << endl

    << xml_endt ("display-parameters") << endl;

    return os ();
};
