/***************************************************************************
                          c_displayparameters.h  -  description
                             -------------------
    begin                : Fri Aug 1 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_DISPLAYPARAMETERS_H
#define C_DISPLAYPARAMETERS_H

#include "iomanip_aa.h"

using namespace N_AminoAcid;

/**
  *@author Cyril Deremble
  */

class C_DisplayParameters
{
public:
    typedef vector <color_t> cluster_colors_t;
    typedef map <E_AminoAcid, color_t> aas_colors_t;

    C_DisplayParameters ();

    cluster_colors_t & clusterColors ()
    {
        return m_ClusterColors_vct;
    };
    color_t & clusterBreakersColor ()
    {
        return m_ClusterBreakersColor;
    };
    aas_colors_t & aasColors ()
    {
        return m_AAsColors_map;
    };
    string & outputStylesheetHRef ()
    {
        return m_OutputStylesheetHRef_str;
    };
    size_t & seqLineWidth ()
    {
        return m_SeqLineWidth_n;
    };

    const cluster_colors_t & clusterColors () const
    {
        return m_ClusterColors_vct;
    };
    const color_t & clusterBreakersColor () const
    {
        return m_ClusterBreakersColor;
    };
    const aas_colors_t & aasColors () const
    {
        return m_AAsColors_map;
    };
    const string & outputStylesheetHRef () const
    {
        return m_OutputStylesheetHRef_str;
    };
    const size_t & seqLineWidth () const
    {
        return m_SeqLineWidth_n;
    };


private:
    cluster_colors_t m_ClusterColors_vct;
    color_t m_ClusterBreakersColor;
    aas_colors_t m_AAsColors_map;
    string m_OutputStylesheetHRef_str;

    size_t m_SeqLineWidth_n;
};

extern C_DisplayParameters g_DisplayParameters;


/**
  *@author Cyril Deremble
  */

class C_DisplayParametersIOManip : public C_IOStreamManip <C_XMLTag, ostream>
{
public:
    C_XMLTag & operator >> (C_DisplayParameters & DisplayParams) const;
    ostream & operator << (const C_DisplayParameters & DisplayParams) const;
};

const C_DisplayParametersIOManip xml_displayparams = C_DisplayParametersIOManip ();

inline const C_DisplayParametersIOManip & operator >> (C_XMLTag & Stream, const C_DisplayParametersIOManip & Manip)
{
    Manip.setIStream (Stream);
    return Manip;
};

inline const C_DisplayParametersIOManip & operator << (ostream & Stream, const C_DisplayParametersIOManip & Manip)
{
    Manip.setOStream (Stream);
    return Manip;
};

#endif
