/***************************************************************************
                          c_filepathname.cpp  -  description
                             -------------------
    begin                : Fri May 23 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "c_filepathname.h"

using namespace N_Tools;

static const char UNKNOWN_DELIM = '?';
static const char EXTENSION_DELIM = '.';
static const string PATH_DELIMS = "/\\";

/***************************************************************************
 *                                                                         *
 *   getPath                                                               *
 *                                                                         *
 ***************************************************************************/

char
C_FilePathName::
getDelimiter () const
{
    size_t FirstDelim_n = find_first_of (PATH_DELIMS);
    return ((FirstDelim_n < size ()) ?
            (*this) [FirstDelim_n] : UNKNOWN_DELIM);
}

/***************************************************************************
 *                                                                         *
 *   getPath                                                               *
 *                                                                         *
 ***************************************************************************/

string
C_FilePathName::
getPath () const
{
    size_t LastDelim_n = find_last_of (PATH_DELIMS);
    return ((LastDelim_n < size ()) ?
            string (begin (), begin () + LastDelim_n + 1) : string ());
}


/***************************************************************************
 *                                                                         *
 *   getName                                                               *
 *                                                                         *
 ***************************************************************************/

string
C_FilePathName::
getName () const
{
    size_t LastDelim_n = find_last_of (PATH_DELIMS);
    return ((LastDelim_n < size ()) ?
            string (begin () + LastDelim_n + 1, end ()) : static_cast <string> (*this));
}


/***************************************************************************
 *                                                                         *
 *   getLastExtension                                                      *
 *                                                                         *
 ***************************************************************************/

string
C_FilePathName::
getLastExtension () const
{
    string Name_str = getName ();
    size_t LastExt_n = Name_str.find_last_of (EXTENSION_DELIM);
    return ((LastExt_n < Name_str.size ()) ?
            string (Name_str.begin () + LastExt_n + 1, Name_str.end ()) : string ());
}

/***************************************************************************
 *                                                                         *
 *   getExtension                                                          *
 *                                                                         *
 ***************************************************************************/

string
C_FilePathName::
getExtension () const
{
    string Name_str = getName ();
    size_t FirstExt_n = Name_str.find_first_of (EXTENSION_DELIM);
    return ((FirstExt_n < Name_str.size ()) ?
            string (Name_str.begin () + FirstExt_n + 1, Name_str.end ()) : string ());
}



/***************************************************************************
 *                                                                         *
 *   getNameWithoutExtension                                               *
 *                                                                         *
 ***************************************************************************/

string
C_FilePathName::
getNameWithoutExtension () const
{
    string Name_str = getName ();
    size_t LastExt_n = Name_str.find_first_of (EXTENSION_DELIM);
    return ((LastExt_n < Name_str.size ()) ?
            string (Name_str.begin (), Name_str.begin () + LastExt_n) : Name_str);
}


/***************************************************************************
 *                                                                         *
 *   getPathNameWithoutExtension                                           *
 *                                                                         *
 ***************************************************************************/

string
C_FilePathName::
getPathNameWithoutExtension () const
{
    size_t LastExt_n = find_last_of (EXTENSION_DELIM);
    return ((LastExt_n < size ()) ?
            string (begin (), begin () + LastExt_n) : static_cast <string> (*this));
}
