/***************************************************************************
                          c_iterator.h  -  description
                             -------------------
    begin                : Fri Jun 13 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_ITERATOR_H
#define C_ITERATOR_H

#include "c_iteratorbase.h"

namespace N_Tools
{

/** base template class for overloading golbal iterators
  *@author Cyril Deremble
  */
template <class value_t, class iterator_t>
class C_Iterator : C_IteratorBase <value_t *>
{
public:
    /** type of the value pointed
      */
    typedef typename C_IteratorBase <value_t *>::value_type value_type;

    /** reference type of the value pointed
      */
    typedef typename C_IteratorBase <value_t *>::reference_type reference_type;

    /** pointer type of the value pointed
      */
    typedef typename C_IteratorBase <value_t *>::pointer_type pointer_type;

    /** type of the based iterator
      */
    typedef iterator_t iterator;

    /** explicitly construct the object
      *@param it based iterator type value
      */
    explicit C_Iterator (const iterator & it = iterator ()) : m_it (it)
    {}
    ;

    /** construct the object from an other iterator
      * (execute conversion between iterators)
      * shall be overrided is derived class
      *@param it iterator of an other type
      */
    template <class T, class U>
    C_Iterator (const C_Iterator <T, U> & it)
            : m_it ((iterator) (U) it)
    {}
    ;

    /** convert to the based iterator type
      */
    operator iterator () const
    {
        return m_it;
    };

    /**
      *@return a reference to the value pointed
      */
    reference_type operator * () const
    {
        return *m_it;
    };


    /** points to the value
      */
    iterator operator -> () const
    {
        return m_it;
    };

    /** increment
      *@return a reference of itself
      */
    C_Iterator & operator ++ ()
    {
        ++ m_it;
        return *this;
    };

    /** increment
      *@return the value of itself before the increment
      */
    C_Iterator operator ++ (int)
    {
        return C_Iterator (m_it ++);
    };

    /** increment
      *@return a reference to itself
      *@param n the increment
      */
    C_Iterator & operator += (int n)
    {
        m_it += n;
        return *this;
    };

    /** decrement
      *@return a reference to itself
      */
    C_Iterator & operator -- ()
    {
        -- m_it;
        return *this;
    };

    /** decrement
      *@return the value of itself before the decrement
      */
    C_Iterator operator -- (int)
    {
        return C_Iterator (m_it --);
    };

    /** decrement
      *@return a reference to itself
      *@param n the decrement
      */
    C_Iterator & operator -= (int n)
    {
        m_it -= n;
        return *this;
    };

    /** comparison operator
      */
    bool operator == (const iterator & Rhs) const
    {
        return m_it == Rhs;
    };
    template <class T, class U>
    bool operator == (const C_Iterator <T, U> & Rhs) const
    {
        return m_it == (U) Rhs;
    };

    /** comparison operator
      */
    bool operator != (const iterator & Rhs) const
    {
        return m_it != Rhs;
    };
    template <class T, class U>
    bool operator != (const C_Iterator <T, U> & Rhs) const
    {
        return m_it != (U) Rhs;
    };


    /** comparison operator
      */
    bool operator < (const iterator & Rhs) const
    {
        return m_it < Rhs;
    };
    template <class T, class U>
    bool operator < (const C_Iterator <T, U> & Rhs) const
    {
        return m_it < (U) Rhs;
    };

    /** comparison operator
      */
    bool operator <= (const iterator & Rhs) const
    {
        return m_it <= Rhs;
    };
    template <class T, class U>
    bool operator <= (const C_Iterator <T, U> & Rhs) const
    {
        return m_it <= (U) Rhs;
    };

    /** comparison operator
      */
    bool operator > (const iterator & Rhs) const
    {
        return m_it > Rhs;
    };
    template <class T, class U>
    bool operator > (const C_Iterator <T, U> & Rhs) const
    {
        return m_it > (U) Rhs;
    };

    /** comparison operator
      */
    bool operator >= (const iterator & Rhs) const
    {
        return m_it >= Rhs;
    };
    template <class T, class U>
    bool operator >= (const C_Iterator <T, U> & Rhs) const
    {
        return m_it >= (U) Rhs;
    };

protected:
    iterator m_it; /// base iterator
};


};

#endif


