/***************************************************************************
                          c_matrix.h  -  description
                             -------------------
    begin                : Thu Jun 12 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_MATRIX_H
#define C_MATRIX_H


/***************************************************************************
 *                                                                         *
 *   Include files                                                         *
 *                                                                         *
 ***************************************************************************/

#include <vector>
#include "c_matrixiterator.h"

namespace N_Tools
{
using namespace std;

struct S_MatSize
{
    size_t N_row;
    size_t M_col;

    S_MatSize (size_t n = 0, size_t m = 0) : N_row (n), M_col (m)
    {}
    ;

    operator size_t () const
    {
        return N_row*M_col;
    };
};


/** class that defines a matrix
  *@author Cyril Deremble
  */
template <class T>
class C_Matrix : public vector <T>
{
public:
    typedef vector <T> base_class_t;
    typedef typename vector <T>::value_type value_type;
    typedef typename vector <T>::iterator iterator;
    typedef typename vector <T>::const_iterator const_iterator;
    typedef typename vector <T>::reverse_iterator reverse_iterator;
    typedef typename vector <T>::const_reverse_iterator const_reverse_iterator;
    typedef C_MatrixIterator <value_type, iterator> mat_iterator;
    typedef C_MatrixIterator <const value_type, const_iterator> mat_const_iterator;
    typedef C_MatrixIterator <value_type, reverse_iterator> mat_reverse_iterator;
    typedef C_MatrixIterator <const value_type, const_reverse_iterator> mat_const_reverse_iterator;
    typedef S_MatSize mat_size_t;

    C_Matrix ()
    {}
    ;

    C_Matrix (const mat_size_t & Size)
            : vector <T> (Size), m_Size (Size)
    {}
    ;

    C_Matrix (const mat_size_t & Size, const T & Value)
            : vector <T> (Size, Value), m_Size (Size)
    {}
    ;

    C_Matrix (const mat_size_t & Size, const T * p)
            : vector <T> (p, p + Size), m_Size (Size)
    {}
    ;

    template <class U>
    C_Matrix (const C_Matrix <U> Mat)
            : vector <T> (Mat.begin (), Mat.end ()), m_Size (Mat.size ())
    {}
    ;

    ~C_Matrix ()
    {}
    ;

    mat_size_t size () const
    {
        return m_Size;
    };
    void resize (size_t n, size_t m)
    {
        m_Size.N_row = n;
        m_Size.M_col = m;
        base_class_t::resize (m_Size);
    };
    void resize (mat_size_t Size)
    {
        m_Size = Size;
        base_class_t::resize (m_Size);
    };

    mat_iterator beginMat ()
    {
        return mat_iterator (base_class_t::begin (), m_Size.M_col);
    };
    mat_const_iterator beginMat () const
    {
        return mat_const_iterator (base_class_t::begin (), m_Size.M_col);
    };

    mat_iterator endMat ()
    {
        return mat_iterator (base_class_t::end (), m_Size.M_col);
    };
    mat_const_iterator endMat () const
    {
        return mat_const_iterator (base_class_t::end (), m_Size.M_col);
    };

    mat_reverse_iterator rbeginMat ()
    {
        return mat_reverse_iterator (base_class_t::rbegin (), m_Size.M_col);
    };
    mat_const_reverse_iterator rbeginMat () const
    {
        return mat_const_reverse_iterator (base_class_t::rbegin (), m_Size.M_col);
    };

    const_iterator operator [] (size_t i) const
    {
        return base_class_t::begin () + i*m_Size.M_col;
    };
    iterator operator [] (size_t i)
    {
        return base_class_t::begin () + i*m_Size.M_col;
    };

private:
    mat_size_t m_Size;
};

/**
  *@author Cyril Deremble
  */

template <class T, int N, int M>
class C_FixedSizeMatrix : public C_Matrix <T>
{
public:
    typedef typename C_Matrix <T>::mat_size_t mat_size_t;

    C_FixedSizeMatrix ()
            : C_Matrix <T> (mat_size_t (N, M))
    {}
    ;

    C_FixedSizeMatrix (const T & Value)
            : C_Matrix <T> (mat_size_t (N, M), Value)
    {}
    ;

    C_FixedSizeMatrix (const T * p)
            : C_Matrix <T> (mat_size_t (N, M), p)
    {}
    ;

    ~C_FixedSizeMatrix ()
    {}
    ;

private:
    using C_Matrix <T>::resize;
    using C_Matrix <T>::capacity;
    using C_Matrix <T>::reserve;
};

};

#endif
