/***************************************************************************
                          c_matrixiterator.h  -  description
                             -------------------
    begin                : Thu Jun 12 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_MATRIXITERATOR_H
#define C_MATRIXITERATOR_H

#include "c_iterator.h"

namespace N_Tools
{

/** iterator object used in the @ref C_Matrix class
  *@author Cyril Deremble
  */
template <class value_t, class iterator_t>
class C_MatrixIterator : public C_Iterator <value_t, iterator_t>
{
public:
    /** base class: @ref C_Iterator
      */
    typedef C_Iterator <value_t, iterator_t> base_class_t;

    /** type of the pointed elements
      */
    typedef value_t value_type;

    /** based iterator type
      */
    typedef iterator_t iterator;

    /** construct explicitly the object
      *@param it based iterator type value
      *@param MCol_n number of column of the matrix pointed
      */
    explicit C_MatrixIterator (const iterator & it = iterator (),
                               size_t MCol_n = size_t  ())
            : base_class_t (it), m_MCol_n (MCol_n)
    {}
    ;

    /** construct the object from an other iterator
      * (execute conversion between iterators)
      * shall be overrided is derived class
      *@param it iterator of an other type
      */
    template <class T, class U>
    C_MatrixIterator (const C_MatrixIterator <T, U> & it)
            : base_class_t (it), m_MCol_n (it.colSize ())
    {}
    ;

    /** destructor
      */
    ~C_MatrixIterator()
    {}
    ;

    /** column increment
      *@return a reference to itself
      */
    C_MatrixIterator & colIncr ()
    {
        ++ base_class_t::m_it;
        return *this;
    };

    /** row increment
      *@return a reference to itself
      */
    C_MatrixIterator & rowIncr ()
    {
        base_class_t::m_it += m_MCol_n;
        return *this;
    };

    /** column increment
      *@return a reference to itself
      *@param n the increment
      */
    C_MatrixIterator & colIncr (size_t n)
    {
        base_class_t::m_it += n;
        return *this;
    };

    /** row increment
      *@return a reference to itself
      *@param n the increment
      */
    C_MatrixIterator & rowIncr (size_t n)
    {
        base_class_t::m_it += n*m_MCol_n;
        return *this;
    };

    /** column decrement
      *@return a reference to itself
      */
    C_MatrixIterator & colDecr ()
    {
        -- base_class_t::m_it;
        return *this;
    };

    /** row decrement
      *@return a reference to itself
      */
    C_MatrixIterator & rowDecr ()
    {
        base_class_t::m_it -= m_MCol_n;
        return *this;
    };

    /** column decrement
      *@return a reference to itself
      *@param n the decrement
      */
    C_MatrixIterator & colDecr (size_t n)
    {
        base_class_t::m_it -= n;
        return *this;
    };

    /** row decrement
      *@return a reference to itself
      *@param n the decrement
      */
    C_MatrixIterator & rowDecr (size_t n)
    {
        base_class_t::m_it -= n*m_MCol_n;
        return *this;
    };

    /** column and row increment
      *@return a reference to itself
      */
    C_MatrixIterator & diagIncr ()
    {
        base_class_t::m_it += m_MCol_n + 1;
        return *this;
    };

    /** column and row decrement
      *@return a reference to itself
      */
    C_MatrixIterator & diagDecr ()
    {
        base_class_t::m_it -= m_MCol_n + 1;
        return *this;
    };

    /** column and row increment
      *@return a reference to itself
      *@param n the increment
      */
    C_MatrixIterator & diagIncr (size_t n)
    {
        base_class_t::m_it += n*(m_MCol_n + 1);
        return *this;
    };

    /** column and row decrement
      *@return a reference to itself
      *@param n the decrement
      */
    C_MatrixIterator & diagDecr (size_t n)
    {
        base_class_t::m_it -= n*(m_MCol_n + 1);
        return *this;
    };

    /**
      *@return the size of the column of the iterator
      */
    size_t colSize () const
    {
        return m_MCol_n;
    };

private:
    /** number of columns of the pointed matrix
      */
    size_t m_MCol_n;
};

};

#endif
