/***************************************************************************
                          c_msfseqimanip.h  -  description
                             -------------------
    begin                : Thu Jul 10 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_MSFSEQIMANIP_H
#define C_MSFSEQIMANIP_H

#include "iomanip_aa.h"
#include "c_seqlstimanip.h"

using namespace N_AminoAcid;

struct S_MsfHeader
    {}
;

const S_MsfHeader msfheader = S_MsfHeader ();

istream & operator >> (istream & is, S_MsfHeader);

struct S_IsClustalAnalysisChar : public unary_function <char, bool>
{
    bool operator () (char c) const
    {
        return s_ClustalAnalysisChars_set.count (c);
    };

private:
    static const set
        <char> s_ClustalAnalysisChars_set;
};

const S_IsClustalAnalysisChar isClustalAnalysisChar = S_IsClustalAnalysisChar ();


class C_MsfSeqIManip : public C_StreamManip <istream>
{
public:
    C_MsfSeqIManip () : m_SeqNb (0)
    {}
    ;

    template <class seq_t>
    istream & operator >> (seq_t & Sequence) const;

    void reset () const
    {
        m_SeqNb = 0;
    };

private:
    template <class seq_t>
    void readSeqLine (seq_t & Sequence) const;

    mutable size_t m_SeqNb;
};

const C_MsfSeqIManip imsfseq = C_MsfSeqIManip ();
const C_SeqLstIManip <C_MsfSeqIManip> imsfseqlst = C_SeqLstIManip <C_MsfSeqIManip> ();

inline const C_MsfSeqIManip & operator >> (istream & Stream, const C_MsfSeqIManip & Manip)
{
    Manip.setStream (Stream);
    return Manip;
};


template <class seq_t>
istream &
C_MsfSeqIManip::
operator >> (seq_t & Sequence) const
{
    streampos BeginSeqPos = s ().tellg ();

    s () >> Sequence.name () >> ws;
    if (! s ().good ())
    {
        s ().clear (ios::failbit | s ().rdstate ());
    }
    else
    {
        while (s ().good ())
        {
            readSeqLine (Sequence);

            while (s ().good ())
            {
                s () >> horws;
                if (! s ().good ())
                    break;
                if (isWhiteSpace (s ().peek ()))
                {
                    s () >> ws;
                    break;
                }
                s () >> line;
            }

            for (size_t i = 0; i < m_SeqNb && s ().good (); i ++)
                s () >> line;
        }
        if (! (s ().fail () || s ().bad ()))
        {
            s ().clear ();
            s ().seekg (BeginSeqPos);
            s () >> ws >> line >> horws;

            if (isWhiteSpace (s ().peek ()) || isClustalAnalysisChar (s ().peek ()))
            {
                s ().seekg (0, ios::end);
                m_SeqNb = 0;
            }
            else
                m_SeqNb ++;
        }
    }

    return s ();
};

template <class seq_t>
void
C_MsfSeqIManip::
readSeqLine (seq_t & Sequence) const
{
    typedef typename seq_t::value_type seq_value_type;

    streampos Pos = s ().tellg ();
    ios::iostate State = s ().rdstate ();
    string Buffer_str;
    s () ;//>> Buffer_str;
    if (Buffer_str != Sequence.name ())
    {
        s ().clear (State);
        s ().seekg (Pos);
    }

    s () >> horws;
    while (s ().good ())
    {
        if (isWhiteSpace (s ().peek ()))
        {
            s ().ignore (1);
            break;
        }

        seq_value_type Val;
        s () >> aa1l >> Val >> horws;
        if (! s ().fail ())
            Sequence.push_back (Val);

        if (s ().eof ())
            s ().clear (ios::eofbit);
    }
};


class C_MsfSeqsIManip : public C_StreamManip <istream>
{
public:
    template <class seqLst_t>
    istream & operator >> (seqLst_t & Sequences_lst) const
    {
        imsfseqlst.m_seq_format.reset ();
        return s () >> msfheader >> imsfseqlst >> Sequences_lst;
    };
};

const C_MsfSeqsIManip imsfseqs = C_MsfSeqsIManip ();

inline const C_MsfSeqsIManip & operator >> (istream & Stream, const C_MsfSeqsIManip & Manip)
{
    Manip.setStream (Stream);
    return Manip;
};

#endif
