/***************************************************************************
                          c_msfseqomanip.h  -  description
                             -------------------
    begin                : Thu Jul 10 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_MSFSEQOMANIP_H
#define C_MSFSEQOMANIP_H

#include <config.h>
#include "iomanip_aa.h"
#include "c_seqlstomanip.h"


using namespace N_AminoAcid;

class C_MsfSeqOManip : public C_StreamManip <ostream>
{
public:
    C_MsfSeqOManip (size_t SeqOffset_n = 0,
                    size_t SeqWidth_n = 0,
                    size_t SeqNameWidth_n = 0)
            : m_SeqOffset_n (SeqOffset_n), m_SeqWidth_n (SeqWidth_n), m_SeqNameWidth_n (SeqNameWidth_n)
    {}
    ;

    template <class seq_t>
    ostream & operator << (const seq_t & Sequence) const
    {
        typedef typename seq_t::const_iterator iterator;
        char LastFill_c = s ().fill (' ');
        int LastWidth = s ().width (m_SeqNameWidth_n);
        s ().setf (ios::left, ios::adjustfield);

        s () << Sequence.name ();

        s ().fill (LastFill_c);
        s ().width (LastWidth);
        s ().setf (ios::right, ios::adjustfield);

        s () << "      ";

        iterator Begin = Sequence.begin () + m_SeqOffset_n;
        size_t Count_n = 0;
        for (iterator i = Begin; i < Sequence.end () && i < (Begin + m_SeqWidth_n); i ++, Count_n ++)
        {
            if ((Count_n % 10) == 0)
                s () << ' ';
            s () << aa1l << *i;
        }
        s () << endl;

        return s ();
    };

    C_MsfSeqOManip operator () (size_t SeqOffset_n, size_t SeqWidth_n, size_t SeqNameWidth_n) const
    {
        return C_MsfSeqOManip (SeqOffset_n, SeqWidth_n, SeqNameWidth_n);
    };

    size_t seqOffset () const
    {
        return m_SeqOffset_n;
    };
    size_t setSeqOffset (size_t SeqOffset_n)
    {
        size_t LastSeqOffset_n = m_SeqOffset_n;
        m_SeqOffset_n = SeqOffset_n;
        return LastSeqOffset_n;
    };

    size_t seqWidth () const
    {
        return m_SeqWidth_n;
    };
    size_t setSeqWidth (size_t SeqWidth_n)
    {
        size_t LastSeqWidth_n = m_SeqWidth_n;
        m_SeqWidth_n = SeqWidth_n;
        return LastSeqWidth_n;
    };

    size_t seqNameWidth () const
    {
        return m_SeqNameWidth_n;
    };
    size_t setSeqNameWidth (size_t SeqNameWidth_n)
    {
        size_t LastSeqNameWidth_n = m_SeqNameWidth_n;
        m_SeqNameWidth_n = SeqNameWidth_n;
        return LastSeqNameWidth_n;
    };

private:
    size_t m_SeqOffset_n;
    size_t m_SeqWidth_n;
    size_t m_SeqNameWidth_n;
};

const C_MsfSeqOManip omsfseq = C_MsfSeqOManip (0, 50, 10);
const C_SeqLstOManip <C_MsfSeqOManip> omsfseqlst = C_SeqLstOManip <C_MsfSeqOManip> ();

inline const C_MsfSeqOManip & operator << (ostream & Stream, const C_MsfSeqOManip & Manip)
{
    Manip.setStream (Stream);
    return Manip;
};


class C_MsfSeqsOManip : public C_StreamManip <ostream>
{
public:
    C_MsfSeqsOManip (size_t SeqWidth_n = 50)
    {
        m_omsfseqlst.m_seq_format.setSeqWidth (SeqWidth_n);
    };

    template <class seqLst_t>
    ostream & operator << (seqLst_t & Sequences_lst) const
    {
        typedef typename seqLst_t::const_iterator iterator;

        if (! Sequences_lst.empty ())
        {
            s () //<< "!!AA_ALIGNMENT 1.0" << endl
            << PACKAGE << ' ' << VERSION << endl << endl
            << "    MSF: " << Sequences_lst.begin ()->size () << " Type: P .." << endl << endl;

            setSeqNameWidth (0);
            for (iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); i ++)
            {
                s () << " Name: " << i->name () << " Len: " << i->size () << endl;
                if (i->name ().size () > seqNameWidth ())
                    setSeqNameWidth (i->name ().size ());
            }

            s () << endl << "//" << endl << endl;

            setSeqOffset (0);
            while (seqOffset () < Sequences_lst.begin ()->size ())
            {
                s () << m_omsfseqlst << Sequences_lst << endl << endl;
                setSeqOffset (seqOffset () + seqWidth ());
            }
        }
        return s ();
    };

    C_MsfSeqsOManip operator () (size_t SeqWidth_n) const
    {
        return C_MsfSeqsOManip (SeqWidth_n);
    };

private:
    mutable C_SeqLstOManip <C_MsfSeqOManip> m_omsfseqlst;

    size_t seqOffset () const
    {
        return m_omsfseqlst.m_seq_format.seqOffset ();
    };
    void setSeqOffset (size_t Offset_n) const
    {
        m_omsfseqlst.m_seq_format.setSeqOffset (Offset_n);
    };

    size_t seqWidth () const
    {
        return m_omsfseqlst.m_seq_format.seqWidth ();
    };
    void setSeqWidth (size_t Offset_n) const
    {
        m_omsfseqlst.m_seq_format.setSeqWidth (Offset_n);
    };

    size_t seqNameWidth () const
    {
        return m_omsfseqlst.m_seq_format.seqNameWidth ();
    };
    void setSeqNameWidth (size_t Offset_n) const
    {
        m_omsfseqlst.m_seq_format.setSeqNameWidth (Offset_n);
    };
};

const C_MsfSeqsOManip omsfseqs = C_MsfSeqsOManip ();

inline const C_MsfSeqsOManip & operator << (ostream & Stream, const C_MsfSeqsOManip & Manip)
{
    Manip.setStream (Stream);
    return Manip;
};

#endif
