/***************************************************************************
                          c_referencevector.h  -  description
                             -------------------
    begin                : Thu Jan 15 2004
    copyright            : (C) 2004 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_REFERENCEVECTOR
#define C_REFERENCEVECTOR

#include <vector>
#include "c_iterator.h"


namespace N_Tools
{
using namespace std;

template <class value_t, class iterator_t>
class C_ReferenceIterator : public C_Iterator <value_t *, iterator_t>
{
public:
    typedef value_t value_type;
    typedef value_t & reference_type;
    typedef value_t * pointer_type;

    typedef C_Iterator <pointer_type, iterator_t> base_class_type;

    C_ReferenceIterator ()
    {}
    ;
    C_ReferenceIterator (iterator_t i) : base_class_type (i)
    {}
    ;

    reference_type operator * () const
    {
        return *(this->base_class_type::operator * ());
    };

    pointer_type operator -> () const
    {
        return this->base_class_type::operator * ();
    };
};

template <class value_t>
class C_ReferenceVector : public vector <value_t *>
{
public:
    typedef value_t value_type;
    typedef value_type & reference_type;
    typedef value_type * pointer_type;
    typedef const value_type & const_reference_type;
    typedef const value_type * const_pointer_type;

    typedef vector <pointer_type> base_class_type;

    typedef C_ReferenceIterator <value_type, typename base_class_type::iterator> iterator;
    typedef C_ReferenceIterator <const value_type, typename base_class_type::const_iterator> const_iterator;

    typedef C_ReferenceIterator <value_type, typename base_class_type::reverse_iterator> reverse_iterator;
    typedef C_ReferenceIterator <const value_type,
    typename base_class_type::const_reverse_iterator> const_reverse_iterator;


    iterator begin ()
    {
        return iterator (base_class_type::begin ());
    };
    const_iterator begin () const
    {
        return const_iterator (base_class_type::begin ());
    };

    iterator end ()
    {
        return iterator (base_class_type::end ());
    };
    const_iterator end () const
    {
        return const_iterator (base_class_type::end ());
    };

    reverse_iterator rbegin ()
    {
        return reverse_iterator (base_class_type::rbegin ());
    };
    const_reverse_iterator rbegin () const
    {
        return const_reverse_iterator (base_class_type::rbegin ());
    };

    reverse_iterator rend ()
    {
        return reverse_iterator (base_class_type::rend ());
    };
    const_reverse_iterator rend () const
    {
        return const_reverse_iterator (base_class_type::rend ());
    };

    reference_type operator [] (size_t n)
    {
        return *(base_class_type::operator [] (n));
    };
    const_reference_type operator [] (size_t n) const
    {
        return *(base_class_type::operator [] (n));
    };

    reference_type at (size_t n)
    {
        return *(base_class_type::at (n));
    };
    const_reference_type at (size_t n) const
    {
        return *(base_class_type::at (n));
    };

    void push_back (reference_type Value)
    {
        base_class_type::push_back (& Value);
    };

private:
    using base_class_type::front;
    using base_class_type::back;
    using base_class_type::insert;
    using base_class_type::swap;
    using base_class_type::pop_back;
    using base_class_type::erase;
    using base_class_type::resize;
};

};


#endif
