/***************************************************************************
                          c_testiterator.h  -  description
                             -------------------
    begin                : Tue May 6 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_TESTITERATOR_H
#define C_TESTITERATOR_H

/**
  *@author Cyril Deremble
  */


/***************************************************************************
 *                                                                         *
 *   Include files                                                         *
 *                                                                         *
 ***************************************************************************/

#include "c_iterator.h"

namespace N_Tools
{

/***************************************************************************
 *                                                                         *
 *   Class declaration: C_TestIterator                                     *
 *                                                                         *
 ***************************************************************************/

template <class unaryTest_t, class value_t, class iterator_t>
class C_TestIterator : public C_Iterator <value_t, iterator_t>
{
public:
    typedef C_Iterator <value_t, iterator_t> base_class_t;
    typedef value_t value_type;
    typedef iterator_t iterator;

    C_TestIterator ()
    {}
    ;
    C_TestIterator (iterator it,
                    iterator Begin_it,
                    iterator End_it,
                    const unaryTest_t & Test = unaryTest_t ())
            : base_class_t (it),
            m_LeftEnd_it (-- Begin_it),
            m_RightEnd_it (End_it),
            test (Test)
    {
        init ();
    };

    C_TestIterator (const C_TestIterator <unaryTest_t, value_t, iterator_t> & it)
            : base_class_t (it),
            m_LeftEnd_it (-- it.begin ()),
            m_RightEnd_it (it.end ()),
            test (it.test)
    {
        init ();
    };

    template <class T, class U, class V>
    C_TestIterator (C_TestIterator <T, U, V> it,
                    const unaryTest_t & Test = unaryTest_t ())
            : base_class_t (it),
            m_LeftEnd_it (-- it.begin ()),
            m_RightEnd_it (it.end ()),
            test (Test)
    {
        init ();
    };

    C_TestIterator & operator ++ ()
    {
        do
            if ((++ m_it) == m_RightEnd_it)
                break;
        while (! test (*m_it))
            ;
        return *this;
    };
    C_TestIterator operator ++ (int)
    {
        C_TestIterator it (*this);
        do
            if ((++ m_it) == m_RightEnd_it)
                break;
        while (! test (*m_it))
            ;
        return it;
    };
    C_TestIterator & operator += (unsigned int n)
    {
        for (unsigned int i = 0; i < n; i ++)
            if ((++ (*this)) == m_RightEnd_it)
                break;
        return *this;
    };

    C_TestIterator & operator -- ()
    {
        do
            if ((-- m_it) == m_LeftEnd_it)
                break;
        while (! test (*m_it))
            ;
        return *this;
    };
    C_TestIterator operator -- (int)
    {
        C_TestIterator it (*this);
        do
            if ((-- m_it) == m_LeftEnd_it)
                break;
        while (! test (*m_it))
            ;
        return it;
    };
    C_TestIterator & operator -= (unsigned int n)
    {
        for (unsigned int i = 0; i < n; i ++)
            if ((-- (*this)) == m_LeftEnd_it)
                break;
        return *this;
    };


    iterator begin () const
    {
        iterator i = m_LeftEnd_it;
        return ++ i;
    };
    iterator end () const
    {
        return m_RightEnd_it;
    };

    template <class T, class U, class V, class W, class X, class Y>
    friend size_t operator - (const C_TestIterator <T, U, V> & Lhs_it,
                              const C_TestIterator <W, X, Y> & Rhs_it);

protected:
    using base_class_t::m_it;
private:
    iterator m_LeftEnd_it;
    iterator m_RightEnd_it;

    void init ()
    {
        if (m_it != m_RightEnd_it)
            if (! test (*m_it))
                ++ (*this);
    };

    using base_class_t::operator ++;
    using base_class_t::operator +=;
    using base_class_t::operator -=;
    using base_class_t::operator --;

public:
    unaryTest_t test;
};


template <class T, class U, class V>
C_TestIterator <T, U, V> operator + (C_TestIterator <T, U, V> Lhs_it, const size_t Dist_n)
{
    for (size_t i = 0; i < Dist_n; i ++)
        ++ Lhs_it;
    return Lhs_it;
};

template <class T, class U, class V, class W, class X, class Y>
size_t operator - (const C_TestIterator <T, U, V> & Lhs_it, const C_TestIterator <W, X, Y> & Rhs_it)
{
    size_t Dist_n = 0;
    C_TestIterator <T, U, V> it (Rhs_it);
    while ((it ++) < Lhs_it.m_it)
        Dist_n ++;

    return Dist_n;
};

};

#endif
