/***************************************************************************
                          c_treenode.h  -  description
                             -------------------
    begin                : Tue Sep 9 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_TREENODE_H
#define C_TREENODE_H

#include <vector>
#include "c_iteratorbase.h"

namespace N_Tools
{
using namespace std;

/**
  *@author Cyril Deremble
  */

template <class value_t>
class C_TreeNode;

/**
  *@author Cyril Deremble
  */

template <class iterator_t>
class C_TreeLeafIterator : C_IteratorBase <iterator_t>
{
public:
    typedef typename C_IteratorBase <iterator_t>::value_type value_type;
    typedef typename C_IteratorBase <iterator_t>::reference_type reference_type;
    typedef typename C_IteratorBase <iterator_t>::pointer_type pointer_type;

    typedef C_TreeLeafIterator child_type;

    C_TreeLeafIterator () : m_TreeNode_p (NULL), m_TreeNodeEnd_p (NULL), m_Child_p (NULL)
    {}
    ;

    C_TreeLeafIterator (const iterator_t & TreeNode_p, const iterator_t & TreeNodeEnd_p)
            : m_TreeNode_p (TreeNode_p), m_TreeNodeEnd_p (TreeNodeEnd_p), m_Child_p (NULL)
    {
        if (m_TreeNode_p != m_TreeNodeEnd_p)
        {
            if (! m_TreeNode_p->isLeaf ())
            {
                m_Child_p = new C_TreeLeafIterator (m_TreeNode_p->childs ().begin (),
                                                    m_TreeNode_p->childs ().end ());
            }
        }
    };

    C_TreeLeafIterator (const C_TreeLeafIterator & It)
    {
        m_TreeNode_p = It.m_TreeNode_p;
        if (It.m_Child_p != NULL)
        {
            m_Child_p = new C_TreeLeafIterator (*It.m_Child_p);
        }
    };

    ~C_TreeLeafIterator ()
    {
        if (m_Child_p != NULL)
        {
            delete m_Child_p;
        }
    };

    C_TreeLeafIterator & operator = (const C_TreeLeafIterator & It)
    {
        m_TreeNode_p = It.m_TreeNode_p;
        if (It.m_Child_p != NULL)
        {
            m_Child_p = new C_TreeLeafIterator (*It.m_Child_p);
        }
        return *this;
    };

    reference_type operator * () const
    {
        return ((m_Child_p == NULL) ? *m_TreeNode_p : **m_Child_p);
    };

    iterator_t operator -> () const
    {
        return ((m_Child_p == NULL) ? m_TreeNode_p : m_Child_p->operator -> ());
    };

    C_TreeLeafIterator & operator ++ ()
    {
        if (m_Child_p == NULL)
        {
            if ((++ m_TreeNode_p) != m_TreeNodeEnd_p)
            {
                if (! m_TreeNode_p->isLeaf ())
                {
                    m_Child_p = new C_TreeLeafIterator (m_TreeNode_p->childs ().begin (),
                                                        m_TreeNode_p->childs ().end ());
                }
            }
        }
        else
        {
            if (++ (*m_Child_p) == C_TreeLeafIterator (m_TreeNode_p->childs ().end (),
                    m_TreeNode_p->childs ().end ()))
            {
                delete m_Child_p;
                m_Child_p = NULL;

                operator ++ ();
            }
        }
        return *this;
    };

    C_TreeLeafIterator operator ++ (int)
    {
        C_TreeLeafIterator Old  (*this);
        operator ++ ();
        return Old;
    };

    bool operator == (const C_TreeLeafIterator & It)
    {
        if (m_TreeNode_p == It.m_TreeNode_p)
        {
            if (m_Child_p != NULL && It.m_Child_p != NULL)
            {
                return (*m_Child_p) == (*It.m_Child_p);
            }
            else
            {
                return m_Child_p == NULL && It.m_Child_p == NULL;
            }
        }
        else
        {
            return false;
        }
    };

    bool operator != (const C_TreeLeafIterator & It)
    {
        return ! operator == (It);
    };

private:
    iterator_t m_TreeNode_p;
    iterator_t m_TreeNodeEnd_p;

    C_TreeLeafIterator * m_Child_p;
};


template <class iterator_t, class tree_node_iterator_t>
class C_TreeLeafValueIterator : public C_IteratorBase <iterator_t>,
            public C_TreeLeafIterator <tree_node_iterator_t>
{
public:
    typedef C_TreeLeafIterator <tree_node_iterator_t> base_class_type;
    typedef typename C_IteratorBase <iterator_t>::value_type value_type;
    typedef typename C_IteratorBase <iterator_t>::reference_type reference_type;
    typedef typename C_IteratorBase <iterator_t>::pointer_type pointer_type;

    C_TreeLeafValueIterator ()
    {}
    ;

    C_TreeLeafValueIterator (const tree_node_iterator_t & TreeNode_p,
                             const tree_node_iterator_t & TreeNodeEnd_p)
            : base_class_type (TreeNode_p, TreeNodeEnd_p)
    {}
    ;

    C_TreeLeafValueIterator (const base_class_type & It)
            : base_class_type (It)
    {}
    ;

    reference_type operator * () const
    {
        return (base_class_type::operator * ()).value ();
    };

    pointer_type operator -> () const
    {
        return &((base_class_type::operator * ()).value ());
    };

    C_TreeLeafValueIterator & operator ++ ()
    {
        base_class_type::operator ++ ();
        return *this;
    };

    C_TreeLeafValueIterator operator ++ (int)
    {
        return base_class_type::operator ++ (0);
    };
};

template <class value_t>
class C_TreeNode
{
public:
    typedef value_t value_type;
    typedef value_t & reference_type;
    typedef value_t * pointer_type;
    typedef const value_t & const_reference_type;
    typedef const value_t * const_pointer_type;

    typedef C_TreeNode <value_type> child_type;
    typedef vector <child_type> childs_type;
    typedef typename childs_type::iterator childs_iterator;
    typedef typename childs_type::const_iterator childs_const_iterator;

    typedef C_TreeLeafIterator <childs_iterator> leaves_iterator;
    typedef C_TreeLeafIterator <childs_const_iterator> leaves_const_iterator;

    typedef C_TreeLeafValueIterator <pointer_type, childs_iterator> leaves_value_iterator;

    typedef C_TreeLeafValueIterator <const_pointer_type, childs_const_iterator> leaves_value_const_iterator;
    
    typedef double distance_type;

    C_TreeNode () : m_Distance (0)
    {}
    ;
    C_TreeNode (value_type Value, distance_type Distance) : m_Value (Value), m_Distance (Distance)
    {}
    ;

    childs_type & childs ()
    {
        return m_Childs_vct;
    };
    const childs_type & childs () const
    {
        return m_Childs_vct;
    };

    reference_type value ()
    {
        return m_Value;
    };
    const_reference_type value () const
    {
        return m_Value;
    };
    
    distance_type & dist ()
    {
        return m_Distance;
    };
    distance_type dist () const
    {
        return m_Distance;
    };

    bool isLeaf () const
    {
        return m_Childs_vct.empty ();
    };
    
    bool isRoot () const
    {
        return m_Distance == 0;
    };
    
    void clear ()
    {
        childs ().clear ();
        m_Distance = 0;
        m_Value = value_type ();
    };
    
    bool empty ()
    {
        return isLeaf () && isRoot ();
    };

    leaves_iterator beginLeaves ()
    {
        return leaves_iterator (m_Childs_vct.begin (), m_Childs_vct.end ());
    };
    leaves_iterator endLeaves ()
    {
        return leaves_iterator (m_Childs_vct.end (), m_Childs_vct.end ());
    };

    leaves_const_iterator beginLeaves () const
    {
        return leaves_const_iterator (m_Childs_vct.begin (), m_Childs_vct.end ());
    };
    leaves_const_iterator endLeaves () const
    {
        return leaves_const_iterator (m_Childs_vct.end (), m_Childs_vct.end ());
    };

    leaves_value_iterator beginLeavesValue ()
    {
        return leaves_value_iterator (m_Childs_vct.begin (), m_Childs_vct.end ());
    };
    leaves_value_iterator endLeavesValue ()
    {
        return leaves_value_iterator (m_Childs_vct.end (), m_Childs_vct.end ());
    };

    leaves_value_const_iterator beginLeavesValue () const
    {
        return leaves_value_const_iterator (m_Childs_vct.begin (), m_Childs_vct.end ());
    };
    leaves_value_const_iterator endLeavesValue () const
    {
        return leaves_value_const_iterator (m_Childs_vct.end (), m_Childs_vct.end ());
    };

private:
    childs_type m_Childs_vct;
    distance_type m_Distance;
    value_type m_Value;
};

};

#endif
