/***************************************************************************
                          c_xmlseqiomanip.h  -  description
                             -------------------
    begin                : Thu Jul 24 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef C_XMLSEQIOMANIP_H
#define C_XMLSEQIOMANIP_H

#include "iomanip_aa.h"
#include "c_seqlstimanip.h"
#include "c_seqlstomanip.h"

using namespace N_AminoAcid;

class C_XMLSeqIManip : public C_StreamManip <istream>
{
	public:
		C_XMLSeqIManip (const string & DefaultAAFormatName_str = "1-letter")
	: m_xml_iaa (DefaultAAFormatName_str)
		{}
		;

		template <class seq_t>
				istream & operator >> (seq_t & Sequence) const
		{
			typedef typename seq_t::value_type value_type;
			C_XMLTag Tag;

			s () >> Tag;
			if (! s ().fail ())
			{
				if (Tag.name != "sequence")
					s ().setstate (ios::failbit);
				else
				{
					string Code_str = Tag.getAttributeValue ("aa-code");
					if (! Code_str.empty ())
						m_xml_iaa.setDefaultAAFormat (Code_str);

					Sequence.name () = Tag.getAttributeValue ("name");
					Sequence.comments () = Tag.getAttributeValue ("comments");

					while (Tag.good ())
					{
						value_type Val;
						Tag >> m_xml_iaa >> Val >> ws;
						if (Tag.fail ())
							s ().setstate (ios::failbit);
						else
							Sequence.push_back (Val);
					}
				}
			}
			return s ();
		};

		string setDefaultAAFormat (const string & FormatName_str)
		{
			return m_xml_iaa.setDefaultAAFormat (FormatName_str);
		};

	private:
		mutable C_XMLAAIFormat m_xml_iaa;
};

const C_XMLSeqIManip xml_iseq = C_XMLSeqIManip ();
const C_SeqLstIManip <C_XMLSeqIManip> xml_iseqlst = C_SeqLstIManip <C_XMLSeqIManip> ();


inline const C_XMLSeqIManip & operator >> (istream & Stream, const C_XMLSeqIManip & Manip)
{
	Manip.setStream (Stream);
	return Manip;
};


template <class aa_format_t>
		class C_XMLSeqOManip : public C_StreamManip <ostream>
{
	public:
		C_XMLSeqOManip (const aa_format_t & aa_format = aa_format_t (), bool WriteAACodeName_b = false)
	: m_xml_oaa (aa_format), m_WriteAACodeName_b (WriteAACodeName_b)
		{
			m_xml_oaa.dontWriteAACodeName ();
		};

		template <class seq_t>
				ostream & operator << (const seq_t & Sequence) const
		{
			typedef typename seq_t::const_iterator const_iterator;

			s () << xml_begt ("sequence", xml_withat) << xml_at ("name", Sequence.name ());
			if (m_WriteAACodeName_b)
				s () << xml_at ("aa-code", m_xml_oaa.aaCodeStr ());
			s () << xml_endats << endl;

			for (const_iterator i = Sequence.begin (); i != Sequence.end (); i ++)
				s () << m_xml_oaa << *i << endl;

			return s () << xml_endt ("sequence") << endl;
		};

		void writeAACodeName ()
		{
			m_WriteAACodeName_b = true;
		};
		void dontWriteAACodeName ()
		{
			m_WriteAACodeName_b = false;
		};

		string aaCodeStr () const
		{
			return m_xml_oaa.aaCodeStr ();
		};

	private:
		mutable aa_format_t m_xml_oaa;
		bool m_WriteAACodeName_b;
};

typedef C_XMLSeqOManip <C_XMLAA1LOFormat> C_XMLSeq1LOManip;
typedef C_XMLSeqOManip <C_XMLAA3LOFormat> C_XMLSeq3LOManip;
const C_XMLSeq1LOManip xml_oseq1l = C_XMLSeq1LOManip (C_XMLAA1LOFormat (), false);
const C_XMLSeq3LOManip xml_oseq3l = C_XMLSeq3LOManip (C_XMLAA3LOFormat (), false);

typedef C_SeqLstOManip <C_XMLSeq1LOManip> C_SeqLst1LOManip;
typedef C_SeqLstOManip <C_XMLSeq3LOManip> C_SeqLst3LOManip;
const C_SeqLst1LOManip xml_oseqlst1l = C_SeqLst1LOManip ();
const C_SeqLst3LOManip xml_oseqlst3l = C_SeqLst3LOManip ();

template <class aa_format_t>
		inline const C_XMLSeqOManip <aa_format_t> & operator << (ostream & Stream, const C_XMLSeqOManip <aa_format_t> & Manip)
{
	Manip.setStream (Stream);
	return Manip;
};


class C_XMLSeqsIManip : public C_StreamManip <istream>
{
	public:
		C_XMLSeqsIManip (const string & DefaultAAFormatName_str = "1-letter")
		{
			m_xml_iseqlst.m_seq_format.setDefaultAAFormat (DefaultAAFormatName_str);
		};

		template <class seqLst_t>
				istream & operator >> (seqLst_t & Sequences_lst) const
		{
			C_XMLTag Tag;
			s () >> xml_gettag ("sequences", true) >> Tag;
			if (! s ().fail ())
			{
				if (Tag.name != "sequences")
					s ().setstate (ios::failbit);
				else
				{
					string Code_str = Tag.getAttributeValue ("aa-code");
					if (! Code_str.empty ())
						m_xml_iseqlst.m_seq_format.setDefaultAAFormat (Code_str);

					Tag.getAttributeStream ("aligned") >> boolalpha >> Sequences_lst.areAligned ();
					if (Sequences_lst.areAligned ()) Tag.getAttributeStream ("align-score") >> Sequences_lst.alignScore ();
					Tag >> m_xml_iseqlst >> Sequences_lst;

					if (Tag.fail ())
						s ().setstate (ios::failbit);
				}
			}
			return s ();
		};

	private:
		mutable C_SeqLstIManip <C_XMLSeqIManip> m_xml_iseqlst;
};

const C_XMLSeqsIManip xml_iseqs = C_XMLSeqsIManip ();

inline const C_XMLSeqsIManip & operator >> (istream & Stream, const C_XMLSeqsIManip & Manip)
{
	Manip.setStream (Stream);
	return Manip;
};



template <class seqlst_format_t>
		class C_XMLSeqsOManip : public C_StreamManip <ostream>
{
	public:
		C_XMLSeqsOManip (size_t SeqLineWidth_n = 100)
	: m_SeqLineWidth_n (SeqLineWidth_n)
		{
			m_seqlst_format.m_seq_format.dontWriteAACodeName ();
		};

		template <class seqLst_t>
				ostream & operator << (const seqLst_t & Sequences_lst) const
		{
			s () << xml_begt ("sequences", xml_withat) << xml_at ("view-width", m_SeqLineWidth_n)
					<< xml_at ("aa-code", m_seqlst_format.m_seq_format.aaCodeStr ());
			if (Sequences_lst.areAligned ())
			{
				s () << xml_at ("aligned", "true") << xml_at ("align-score", Sequences_lst.alignScore ());
			}
			return s () << xml_endats << endl
					<< m_seqlst_format << Sequences_lst << xml_endt ("sequences") << endl;
		};

		size_t setSeqLineWidth (size_t SeqLineWidth_n)
		{
			size_t LastSeqLineWidth_n = m_SeqLineWidth_n;
			m_SeqLineWidth_n = SeqLineWidth_n;
			return LastSeqLineWidth_n;
		}

	private:
		seqlst_format_t m_seqlst_format;

		size_t m_SeqLineWidth_n;
};

typedef C_XMLSeqsOManip <C_SeqLst1LOManip> C_XMLSeqs1LOManip;
typedef C_XMLSeqsOManip <C_SeqLst3LOManip> C_XMLSeqs3LOManip;
const C_XMLSeqs1LOManip xml_oseqs1l = C_XMLSeqs1LOManip (100);
const C_XMLSeqs3LOManip xml_oseqs3l = C_XMLSeqs3LOManip (100);

template <class seqlst_format_t>
		inline const C_XMLSeqsOManip <seqlst_format_t> & operator << (ostream & Stream, const C_XMLSeqsOManip <seqlst_format_t> & Manip)
{
	Manip.setStream (Stream);
	return Manip;
};


#endif
