/***************************************************************************
                          iomanip_aa.h  -  description
                             -------------------
    begin                : Wed Jul 9 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef IOMANIP_AA_H
#define IOMANIP_AA_H

#include "iomanip_ext.h"
#include "n_aminoacid.h"

namespace N_AminoAcid
{

template <int N>
class C_AAFormat : public C_IOStreamManip <istream, ostream>
{
public:
    template <class T>
    istream & operator >> (T & t) const
    {
        bool Failbit = false;
        char Buffer_str [N + 1];
        Buffer_str [N] = '\0';
        if (is ().flags () & ios::skipws)
            is () >> ws;
        if (! is ().good ())
            is ().setstate (ios::failbit);
        else
        {
            streampos Pos = is ().tellg ();
            is ().read (Buffer_str, N);

            capitalize (Buffer_str);
            if (N == 1)
                t = convertAA <T> (Buffer_str [0]);
            else if (N == 3)
                t = convertAA <T> (string (Buffer_str), Failbit);
            else
                is ().setstate (ios::failbit);

            if (is ().fail () || Failbit)
            {
                is ().clear ();
                is ().seekg (Pos);
                is ().clear (ios::failbit);
            }
        }
        return is ();
    };

    template <class T>
    ostream & operator << (const T & t) const
    {
        if (N == 1)
            os () << convertAA <char> (t);
        else if (N == 3)
            os () << convertAA <string> (t);
        else
            os ().setstate (ios::failbit);
        return os ();
    };

    static string aaCodeStr ()
    {
        return s_FormatName_str;
    };

private:
    static const string s_FormatName_str;
};

typedef C_AAFormat <1> C_AA1LFormat;
typedef C_AAFormat <3> C_AA3LFormat;

const C_AA1LFormat aa1l = C_AA1LFormat ();
const C_AA3LFormat aa3l = C_AA3LFormat ();


template <int N>
inline const C_AAFormat <N> & operator >> (istream & Stream, const C_AAFormat <N> & Manip)
{
    Manip.setIStream (Stream);
    return Manip;
};

template <int N>
inline const C_AAFormat <N> & operator << (ostream & Stream, const C_AAFormat <N> & Manip)
{
    Manip.setOStream (Stream);
    return Manip;
};


class C_AAStrFormat : public C_IOStreamManip <istream, ostream>
{
public:
    template <class T>
    istream & operator >> (T & t) const
    {
        string Buffer_str;
        bool Failbit = false;
        if (! is ().good ())
            is ().clear (ios::failbit | is ().rdstate ());
        else
        {
            streampos Pos = is ().tellg ();
            is () >> word >> Buffer_str;
            capitalize (Buffer_str);
            t = convertStr2AA (Buffer_str, Failbit);
            if (is ().fail () || Failbit)
            {
                is ().clear ();
                is ().seekg (Pos);
                is ().clear (ios::failbit);
            }
        }
        return is ();
    };

    template <class T>
    ostream & operator << (const T & t) const
    {
        return os () << convertAA2Str (convertAA <E_AminoAcid> (t));
    };

    static string aaCodeStr ()
    {
        return s_FormatName_str;
    };

private:
    static const string s_FormatName_str;
};

const C_AAStrFormat aastr = C_AAStrFormat ();

inline const C_AAStrFormat & operator >> (istream & Stream, const C_AAStrFormat & Manip)
{
    Manip.setIStream (Stream);
    return Manip;
};

inline const C_AAStrFormat & operator << (ostream & Stream, const C_AAStrFormat & Manip)
{
    Manip.setOStream (Stream);
    return Manip;
};


class C_XMLAAIFormat : public C_StreamManip <istream>
{
public:
    C_XMLAAIFormat (const string & DefaultAAFormatName_str = "1-letter")
            : m_DefaultFormatName_str (DefaultAAFormatName_str)
    {}
    ;

    template <class T>
    istream & operator >> (T & t) const
    {
        C_XMLTag Tag;
        s () >> Tag;
        if (! s ().fail ())
            if (Tag.name != "aa")
                s ().setstate (ios::failbit);
            else
            {
                string Code_str = Tag.getAttributeValue ("code");
                if (Code_str == "")
                    Code_str = m_DefaultFormatName_str;

                if (Code_str == aa1l.aaCodeStr ())
                    Tag >> aa1l >> t;
                else if (Code_str == aa3l.aaCodeStr ())
                    Tag >> aa3l >> t;
                else if (Code_str == aastr.aaCodeStr ())
                    Tag >> aastr >> t;
                else
                    s ().setstate (ios::failbit);

                if (Tag.fail ())
                    s ().setstate (ios::failbit);
                if (Tag.good ())
                    readAAAttributesFrom (Tag, t);
            }
        return s ();
    };

    string setDefaultAAFormat (const string & FormatName_str)
    {
        string LastName_str = m_DefaultFormatName_str;
        m_DefaultFormatName_str = FormatName_str;
        return LastName_str;
    };

private:
    string m_DefaultFormatName_str;

    void readAAAttributesFrom (C_XMLTag &, E_AminoAcid &) const
        {}
    ;
    void readAAAttributesFrom (C_XMLTag &, char &) const
        {}
    ;
    void readAAAttributesFrom (C_XMLTag &, string &) const
        {}
    ;
    void readAAAttributesFrom (C_XMLTag &, char *) const
        {}
    ;
    void readAAAttributesFrom (C_XMLTag & Tag, analysedAA_t & AA) const
    {
        Tag.getAttributeStream ("conserved-score") >> AA.ConservedScore;
        Tag.getAttributeStream ("cluster-id") >> AA.ClusterId;
        Tag.getAttributeStream ("color") >> htmlcolor >> AA.Color;
    };
};

const C_XMLAAIFormat xml_iaa = C_XMLAAIFormat ("1-letter");

inline const C_XMLAAIFormat & operator >> (istream & Stream, const C_XMLAAIFormat & Manip)
{
    Manip.setStream (Stream);
    return Manip;
};


template <class aa_format_t>
class C_XMLAAOFormat : public C_StreamManip <ostream>
{
public:
    C_XMLAAOFormat (const aa_format_t & aa_format = aa_format_t (), bool WriteAACodeName_b = false)
            : m_aa_format (aa_format), m_WriteAACodeName_b (WriteAACodeName_b)
    {}
    ;

    template <class T>
    ostream & operator << (const T & t) const
    {
        s () << xml_begt ("aa", xml_withat);
        writeAttributes (t);
        s () << xml_endats << m_aa_format << t;
        if (m_aa_format.aaCodeStr () == "3-letter")
            s () << ' ';
        return s () << xml_endt ("aa");
    };

    string aaCodeStr () const
    {
        return m_aa_format.aaCodeStr ();
    };

    void writeAACodeName ()
    {
        m_WriteAACodeName_b = true;
    };
    void dontWriteAACodeName ()
    {
        m_WriteAACodeName_b = false;
    };

protected:
    aa_format_t m_aa_format;

private:
    bool m_WriteAACodeName_b;

    void writeAttributes (const E_AminoAcid &) const
        {}
    ;
    void writeAttributes (const char &) const
        {}
    ;
    void writeAttributes (const string &) const
        {}
    ;
    void writeAttributes (const char *) const
        {}
    ;
    void writeAttributes (const analysedAA_t & AA) const
    {
        if (m_WriteAACodeName_b)
            s () << xml_at ("code", aaCodeStr ());
        if (isInCluster (AA))
            s () << xml_at ("cluster-id", AA.ClusterId);
        if (AA.ClusterColor != WHITE)
            s () << xml_begat ("cluster-color") << htmlcolor << AA.ClusterColor << xml_endat;
        if (AA.Color != BLACK)
            s () << xml_begat ("color") << htmlcolor << AA.Color << xml_endat;
        if (AA.ConservedScore != 0)
            s () << xml_at ("conserved-score", AA.ConservedScore);
    };
};

typedef C_XMLAAOFormat <C_AA1LFormat> C_XMLAA1LOFormat;
typedef C_XMLAAOFormat <C_AA3LFormat> C_XMLAA3LOFormat;
const C_XMLAA1LOFormat xml_oaa1l = C_XMLAA1LOFormat (C_AA1LFormat (), false);
const C_XMLAA3LOFormat xml_oaa3l = C_XMLAA3LOFormat (C_AA3LFormat (), false);

template <class aa_format_t>
inline const C_XMLAAOFormat <aa_format_t> & operator << (ostream & Stream, const C_XMLAAOFormat <aa_format_t> & Manip)
{
    Manip.setStream (Stream);
    return Manip;
};


};

#endif
