/***************************************************************************
                          iomanip_aaseq.h  -  description
                             -------------------
    begin                : Wed Jul 9 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef IOMANIP_AASEQ_H
#define IOMANIP_AASEQ_H

#include <fstream>
#include <iomanip>
#include "iomanip_ext.h"
#include "iomanip_aa.h"
#include "c_fastaseqimanip.h"
#include "c_fastaseqomanip.h"
#include "c_msfseqimanip.h"
#include "c_msfseqomanip.h"
#include "c_foldfitseqiomanip.h"
#include "c_xmlseqiomanip.h"
#include "c_displayparameters.h"


enum E_SeqFileFormat
{
	FASTA = 0,
	MSF,
	FOLDFIT,
	XML,
	XML_3L
};

class C_ISeqFile : public ifstream
{
	public:
		C_ISeqFile ()
		{}
		;
		C_ISeqFile (const string & FileName_str, openmode OpenMode = ios::in)
	: ifstream (FileName_str.c_str (), OpenMode)
		{}
		;

		void open (const string & FileName_str, openmode OpenMode = ios::in)
		{
			ifstream::open (FileName_str.c_str (), OpenMode);
		};

		template <class seqLst_t>
				C_ISeqFile & readSequences (seqLst_t & Sequences_lst);
};


template <class seqLst_t>
		C_ISeqFile &
C_ISeqFile::
		readSequences (seqLst_t & Sequences_lst)
{
	string Buffer_str;
	int n;
	char c;

	clear ();
	seekg (0, beg);

	(*this) >> Buffer_str >> line >> c;
	clear ();
	seekg (0, beg);


	(*this) >> n >> n;
	bool Failbit = fail ();
	clear ();
	seekg (0, beg);

	if (Buffer_str.size () < 1)
		setstate (ios::failbit);
	else if (Buffer_str == "!!AA_MULTIPLE_ALIGNMENT" || Buffer_str == "CLUSTAL" || Buffer_str == "PileUp" || ! Failbit)
	{
		(*this) >> imsfseqs >> Sequences_lst;
		Sequences_lst.areAligned () = true;
	}
	else if (Buffer_str == "<?xml")
	{
		(*this) >> xml_iseqs >> Sequences_lst;
	}
	else if (Buffer_str [0] == '>')
	{
		if (c != '>')
		{
			(*this) >> ifastaseqs >> Sequences_lst;
			Sequences_lst.areAligned () = false;
		}
		else
		{
			(*this) >> ifoldfitseqs >> Sequences_lst;
			Sequences_lst.areAligned () = true;
		}
	}
	else
		setstate (ios::failbit);

	return *this;
};


class C_OSeqFile : public ofstream
{
	public:
		C_OSeqFile ()
		{}
		;
		C_OSeqFile (const string & FileName_str,
			    E_SeqFileFormat Format_e = XML,
			    openmode OpenMode = ios::out)
				: ofstream (FileName_str.c_str (), OpenMode), m_Format_e (Format_e)
		{
			m_XMLHeader.DOCTYPE_str = "sequences";
		};

		void open (const string & FileName_str, openmode OpenMode = ios::out)
		{
			ofstream::open (FileName_str.c_str (), OpenMode);
		};
		
		
		template <class seqLst_t> C_OSeqFile & writeSequences (const seqLst_t & Sequences_lst)
		{
			m_xml_oseqs1l.setSeqLineWidth (g_DisplayParameters.seqLineWidth ());
			m_xml_oseqs3l.setSeqLineWidth (g_DisplayParameters.seqLineWidth ());
			
			switch (m_Format_e)
			{
				case FASTA:
					(*this) << ofastaseqs << Sequences_lst;
					break;
				case FOLDFIT:
					(*this) << ofoldfitseqs << Sequences_lst;
					break;
				case MSF:
					(*this) << omsfseqs << Sequences_lst;
					break;
				case XML:
					(*this) << m_xml_oseqs1l << Sequences_lst;
					break;
				case XML_3L:
					(*this) << m_xml_oseqs3l << Sequences_lst;
					break;
				default:
					setstate (failbit);
			};
			return (*this);
		};
		
		void start ()
		{
			if (m_Format_e == XML || m_Format_e == XML_3L)
			{
				m_XMLHeader.XSLStylesheet_str = g_DisplayParameters.outputStylesheetHRef ();
				(*this) << m_XMLHeader << xml_begt ("hcaal") << '\n';
			}
		}
		
		void stop ()
		{
			if (m_Format_e == XML || m_Format_e == XML_3L)
				(*this) << xml_endt ("hcaal") << '\n';
		}
		
		S_XMLHeader & xmlHeader ()
		{
			return m_XMLHeader;
		};
		S_XMLHeader xmlHeader () const
		{
			return m_XMLHeader;
		};
		
	private:
		E_SeqFileFormat m_Format_e;
		
		C_XMLSeqs1LOManip m_xml_oseqs1l;
		C_XMLSeqs3LOManip m_xml_oseqs3l;
		
		S_XMLHeader m_XMLHeader;
};

#endif
