/***************************************************************************
                          iomanip_ext.h  -  description
                             -------------------
    begin                : Tue Jul 8 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef IOMANIP_EXT_H
#define IOMANIP_EXT_H

#include <istream>
#include <set>
#include "s_are.h"

/**@file iomanip_ext.h
  *@author Cyril Deremble
  */


namespace N_Tools
{

using namespace std;

/**
  *@author Cyril Deremble
  */


const string Caps_str ("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
const string NoCaps_str ("abcdefghijklmnopqrstuvwxyz");

inline void capitalize (char & c)
{
    size_t i = NoCaps_str.find (c);
    if (i < Caps_str.size ())
        c = Caps_str.at (i);
};

inline char getCapitalize (const char c)
{
    size_t i = NoCaps_str.find (c);
    return (i < Caps_str.size ()) ? Caps_str.at (i) : c;
};

inline void capitalize (string & str)
{
    for (string::iterator i = str.begin (); i != str.end (); i ++)
        capitalize (*i);
};

inline string getCapitalize (const string & str)
{
    string Result_str;
    for (string::const_iterator i = str.begin (); i != str.end (); i ++)
        Result_str += getCapitalize (*i);
    return Result_str;
};

inline void capitalize (char * str)
{
    char * p = str;
    while (*p != '\0')
        capitalize (*(p ++));
};

struct S_IsHorWhiteSpace : public unary_function <char, bool>
{
    bool operator () (char c) const
    {
        return s_HorWhiteSpaces_set.count (c);
    };

private:
    static const set
        <char> s_HorWhiteSpaces_set;
};

const S_IsHorWhiteSpace isHorWhiteSpace = S_IsHorWhiteSpace ();

const S_Are <S_IsHorWhiteSpace> areHorWhiteSpaces = S_Are <S_IsHorWhiteSpace> ();

struct S_IsWhiteSpace : public unary_function <char, bool>
{
    bool operator () (char c) const
    {
        return s_WhiteSpaces_set.count (c);
    };

private:
    static const set
        <char> s_WhiteSpaces_set;
};

const S_IsWhiteSpace isWhiteSpace = S_IsWhiteSpace ();

const S_Are <S_IsWhiteSpace> areWhiteSpaces = S_Are <S_IsWhiteSpace> ();

struct S_IsAlpha : public unary_function <char, bool>
{
    bool operator () (char c) const
    {
        return s_Alpha_set.count (c);
    };

private:
    static const set
        <char> s_Alpha_set;
};

const S_IsAlpha isAlpha = S_IsAlpha ();

const S_Are <S_IsAlpha> areAlpha = S_Are <S_IsAlpha> ();

struct S_Delim
{
    explicit S_Delim (char Delim) : m_Delim (Delim)
    {}
    ;

    S_Delim operator () (char Delim) const
    {
        return S_Delim (Delim);
    };

    char delim () const
    {
        return m_Delim;
    };

private:
    char m_Delim;
};

const S_Delim line = S_Delim ('\n');

inline istream & operator >> (istream & is, const S_Delim & Delim)
{
    while (is.good ())
        if (is.get () == Delim.delim ())
            break;
        else if (is.eof ())
            is.clear (ios::eofbit);
    return is;
};

struct S_StrictDelim : public S_Delim
{
    explicit S_StrictDelim (char Delim) : S_Delim (Delim)
    {}
    ;
};

inline istream & operator >> (istream & is, const S_StrictDelim & Delim)
{
    while (is.get () != Delim.delim ())
        if (is.fail ())
            break;
    return is;
};

const S_StrictDelim strictline = S_StrictDelim ('\n');

struct S_HorWS
{}
;

const S_HorWS horws = S_HorWS ();

inline istream & operator >> (istream & is, S_HorWS)
{
    while (is.good ())
        if (isHorWhiteSpace (is.peek ()))
            is.ignore (1);
        else if (is.eof ())
            is.clear (ios::eofbit);
        else
            break;
    return is;
};

template <class T>
class C_StreamManip
{
public:
    typedef T stream_t;

    void setStream (stream_t & Stream) const
    {
        m_Stream_p = &Stream;
    };

    stream_t & s () const
    {
        return *m_Stream_p;
    };

protected:
    mutable stream_t * m_Stream_p;
};


template <class istream_t, class ostream_t>
class C_IOStreamManip : private C_StreamManip <istream_t>, private C_StreamManip <ostream_t>
{
public:
    typedef C_StreamManip <istream_t> ismanip_t;
    typedef C_StreamManip <ostream_t> osmanip_t;

    void setIStream (istream_t & is) const
    {
        ismanip_t::setStream (is);
    };
    istream_t & is () const
    {
        return ismanip_t::s ();
    };

    void setOStream (ostream_t & os) const
    {
        osmanip_t::setStream (os);
    };
    ostream_t & os () const
    {
        return osmanip_t::s ();
    };
};

class C_GetLineIManip : public C_StreamManip <istream>, public S_Delim
{
public:
    explicit C_GetLineIManip (char Delim) : S_Delim (Delim)
    {}
    ;

    C_GetLineIManip operator () (char Delim) const
    {
        return C_GetLineIManip (Delim);
    };

    template <class T>
    istream & operator >> (T & t) const
    {
        char c;
        if (s ().good ())
            t.clear ();
        while (s ().good ())
            if ((c = s ().get ()) == delim ())
                break;
            else
                t.push_back (c);
        return s ();
    };
};

const C_GetLineIManip getLine = C_GetLineIManip ('\n');

inline const C_GetLineIManip & operator >> (istream & Stream, const C_GetLineIManip & Manip)
{
    Manip.setStream (Stream);
    return Manip;
};

class C_GetStrictLineIManip : public C_GetLineIManip
{
public:
    explicit C_GetStrictLineIManip (char Delim) : C_GetLineIManip (Delim)
    {}
    ;

    C_GetStrictLineIManip operator () (char Delim) const
    {
        return C_GetStrictLineIManip (Delim);
    };

    template <class T>
    istream & operator >> (T & t) const
    {
        char c;
        if (s ().good ())
            t.clear ();
        while ((c = s ().get ()) != delim ())
            if (s ().fail ())
                break;
            else
                t.push_back (c);
        return s ();
    };
};

const C_GetStrictLineIManip getStrictLine = C_GetStrictLineIManip ('\n');

inline const C_GetStrictLineIManip & operator >> (istream & Stream, const C_GetStrictLineIManip & Manip)
{
    Manip.setStream (Stream);
    return Manip;
};

class C_WordIManip : public C_StreamManip <istream>
{
public:
    istream & operator >> (string & str) const
    {
        if (s ().flags () & ios::skipws)
            s () >> ws;
        if (! isAlpha (s ().peek ()))
            s ().clear (ios::failbit | s ().rdstate ());
        else
        {
            str.clear ();
            while (s ().good ())
                if (! (isAlpha (s ().peek ()) || s ().peek () == '-'))
                    break;
                else
                    str.push_back (static_cast <char> (s ().get ()));
        }
        return s ();
    };
};

const C_WordIManip word = C_WordIManip ();

inline const C_WordIManip & operator >> (istream & Stream, const C_WordIManip & Manip)
{
    Manip.setStream (Stream);
    return Manip;
};

};

#endif
