/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Tue Apr 15 12:22:51 CEST 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#ifdef HAVE_CONFIG_H
#include <config.h>
#endif



#include <iostream>
using namespace std;

#include "_inputparams.h"
#include "c_scoreconservedres.h"
#include "c_analysedseqs.h"
#include "c_treeimanip.h"
#include "c_treeomanip.h"
#include "c_treenode.h"

list <C_FilePathName> g_InputFileNames_lst;
C_FilePathName g_OutputFileName_str;
E_SeqFileFormat g_OutputFileFormat_e = MSF;
C_FilePathName g_OutputParamFileName_str;
C_FilePathName g_InputTreeFileName_str;
C_FilePathName g_OutputDistMatFileName_str;

bool ClearClusters_b = false;
bool ClearGaps_b = false;
bool ClearColors_b = false;
bool ConstructClustersFromIds_b = false;
bool AnalyseClusters_b = true;
bool FillClusters_b = true;
bool TargetDist_b = false;
bool Align_b = true;
bool AlignPairwiseToTarget_b = false;
bool GlobalLocalAlign_b = false;
bool ScoreConservedRes_b = true;
bool ColorClusters_b = true;
bool ColorClusterBreakers_b = true;
bool ColorAAs_b = true;

void colorClusters (C_AnalysedSeqs & Sequences_lst);
void colorClusterBreakers (C_AnalysedSeqs & Sequences_lst);
void colorAAs (C_AnalysedSeqs & Sequences_lst);


/***************************************************************************
 *                                                                         *
 *   main                                                                  *
 *                                                                         *
 ***************************************************************************/

int main (int argc, char *argv [])
{
	cout << endl << "\t**** "<< PACKAGE << " " << VERSION << " ****" << endl
			<< " Proteins with HYdrophobic Blocks ALignment." << endl << endl;
	
	try
	{
		readInputParams (argc - 1, argv + 1);
		
		if (g_OutputFileFormat_e != XML && g_OutputFileFormat_e != XML_3L)
		{
			ScoreConservedRes_b = false;
			ColorClusters_b = false;
			ColorClusterBreakers_b = false;
			ColorAAs_b = false;
		}
		
		C_AnalysedSeqs Sequences_lst;

		vector <C_AnalysedSeqs> PairSequences_vct;
		C_TreeNode <string> Tree;
		
		C_ParameterFile ParamFile;
		for (list <C_FilePathName>::iterator i = g_InputFileNames_lst.begin ();
				   i != g_InputFileNames_lst.end (); i ++)
		{
			ParamFile.clear ();
			ParamFile.open (*i, ios::in);
			if (ParamFile.isParameterFile ())
			{
				cout << "read parameter file '" << *i << "': ";
				ParamFile.readParameters ();
				if (ParamFile.fail ())
					cout << " failed" << endl;
				else
					cout << "ok" << endl;
				i = g_InputFileNames_lst.erase (i);
				i --;
			}
			ParamFile.close ();
		}
		
		if (GlobalLocalAlign_b)
		  clustAlign.setGlobalLocal ();
		
		C_ISeqFile SeqFile;
		for (list <C_FilePathName>::const_iterator i = g_InputFileNames_lst.begin ();
				   i != g_InputFileNames_lst.end (); i ++)
		{
			SeqFile.clear ();
			SeqFile.open (*i);
			cout << "read sequence file '" << *i << "': ";
			SeqFile.readSequences (Sequences_lst);
			if (SeqFile.fail ())
				cout << " failed" << endl;
			else
				cout << "ok" << endl;
			SeqFile.close ();
		}

		if (Sequences_lst.empty ())
		{
			AnalyseClusters_b = false;
			Align_b = false;
			ColorClusters_b = false;
			ColorClusterBreakers_b = false;
			ColorAAs_b = false;
		}
		cout << '\t' << Sequences_lst.size () << " sequence" << ((Sequences_lst.size () > 1) ? "s ": " ") << "read" << endl;
		
		if (! Sequences_lst.areAligned ())
		{
			for (C_AnalysedSeqs::iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); i ++)
				i->clearConservedScore ();
		}
		
		if (ClearGaps_b)
		{
			cout << "clear gaps: ";
			Sequences_lst.areAligned () = false;
			for (C_AnalysedSeqs::iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); i ++)
				i->remove_if <S_IsGap> ();
			cout << "ok" << endl;
		}
		if (ClearClusters_b)
		{
			cout << "clear blocks: ";
			for (C_AnalysedSeqs::iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); i ++)
				i->clearClusters ();
			cout << "ok" << endl;
		}
		if (ClearColors_b)
		{
			cout << "clear blocks: ";
			for (C_AnalysedSeqs::iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); ++ i)
				i->clearColors ();
			cout << "ok" << endl;
		}
		if (ConstructClustersFromIds_b)
		{
			cout << "construct hydrophobic blocks from input ids: ";
			for (C_AnalysedSeqs::iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); ++ i)
			{
				i->constructClustersFromIds ();
			}
			cout << "ok" << endl;
		}
		if (AnalyseClusters_b)
		{
			cout << "analyse hydrophobic blocks: ";
			for (C_AnalysedSeqs::iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); ++ i)
			{
				analyseClusters (*i);
			}
			cout << "ok" << endl;
		}
		if (FillClusters_b)
		{
			cout << "fill hydrophobic blocks: ";
			for (C_AnalysedSeqs::iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); ++ i)
			{
				i->fillClusters ();
			}
			cout << "ok" << endl;
		}
			cout << "set blocks ids: ";
			for (C_AnalysedSeqs::iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); ++ i)
			{
				i->setClusterIds ();
			}
			cout << "ok" << endl;
		if (! g_InputTreeFileName_str.empty ())
		{
			ifstream InputTreeFile;
			cout << "read tree file '" << g_InputTreeFileName_str << "': ";
			InputTreeFile.open (g_InputTreeFileName_str.c_str (), ios::in);
			InputTreeFile >> itree >> Tree;
			if (InputTreeFile.fail ())
			{
				Tree.clear ();
				cout << " failed" << endl;
			}
			else
			{
				cout << "ok" << endl;
			}
	
			InputTreeFile.close ();
		}
		// distance matrice and pairwise alignment for scoredist
		if (! g_OutputDistMatFileName_str.empty () && Sequences_lst.size () > 1)
		{
			ofstream OutputDistMatFile;
			OutputDistMatFile.open (g_OutputDistMatFileName_str.c_str ());
			if (! OutputDistMatFile.is_open ())
			{
				cout << "error, cannot open output file for the distance matrix '" << g_OutputDistMatFileName_str << "'!\n";
			}
			else
			{
			        string bef = "\t";
			        OutputDistMatFile << Sequences_lst.size () << '\n';
				for (C_AnalysedSeqs::const_iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); i ++)
				{
					OutputDistMatFile << setw (10) << left << i->name ();
					for (C_AnalysedSeqs::const_iterator j = i ; j != Sequences_lst.end (); j ++)
					{
					  OutputDistMatFile << bef << clustAlign.getAlignScore (*i, *j);
					  PairSequences_vct.push_back (C_AnalysedSeqs ());
					  PairSequences_vct.back ().push_back (*i);
					  PairSequences_vct.back ().push_back (*j);
					  PairSequences_vct.back ().align ();
					}
					bef += '\t';
					OutputDistMatFile << '\n';
				}
			}
		}
		// alignpairwise for detection
		if (AlignPairwiseToTarget_b)
		{
			if (Sequences_lst.size () > 1)
			{
			        if (g_OutputFileName_str.empty ())
				    g_OutputFileName_str = "out.msf";
				C_OSeqFile OSeqFile (g_OutputFileName_str, g_OutputFileFormat_e);
				OSeqFile.start ();
				C_AnalysedSeqs::const_iterator Target = Sequences_lst.begin ();
				cout << "align sequences pairwise with " << Target->name () << ":\n";

				for (C_AnalysedSeqs::const_iterator i = ++ Sequences_lst.begin (); i != Sequences_lst.end (); ++ i)
				{
				        PairSequences_vct.push_back (C_AnalysedSeqs ());
					PairSequences_vct.back ().push_back (*Target);
					PairSequences_vct.back ().push_back (*i);
					
					if (float(PairSequences_vct.back().front().size())/float(PairSequences_vct.back().back().size()) <=0.55 or float(PairSequences_vct.back().back().size())/float(PairSequences_vct.back().front().size()) <=0.55 ){
					    clustAlign.setGlobalLocal();
					    cout << " * GLOBAL-LOCAL ALGORITHM SET * " << endl; 
					}
					else{
					    clustAlign.unsetGlobalLocal();
					    cout << " * GLOBAL ALGORITHM SET * " << endl; 
					}
					
					cout << '\t' << i->name () << ": " << PairSequences_vct.back ().align () << '\n';
					
					for (vector <C_AnalysedSeqs>::const_iterator i = PairSequences_vct.begin (); i != PairSequences_vct.end (); ++ i)
					{
					    OSeqFile.writeSequences (*i);
					}
					if (OSeqFile.fail ())
					    cout << "failed" << endl;
					PairSequences_vct.erase (PairSequences_vct.begin ());

				}
				cout << "\tok" << endl;
				cout << "write sequences to out file '" << g_OutputFileName_str << "': ";
				OSeqFile.stop ();
				if (OSeqFile.fail ())
				    cout << "failed" << endl;
				else
				    cout << "ok" << endl;	
				
			}

		}
		// multiple alignment
		if (Align_b)
		{
		        cout << "align sequences: " << endl;
			cout << "\tscore = " << (Tree.empty () ? Sequences_lst.align () : Sequences_lst.align (Tree)) << endl;
			cout << "\tok" << endl;
			
			//iterative refinement
			//cout << "iterative refinement: ";
			//C_AnalysedSeqs SequencesIter_lst = Sequences_lst.clone();
			//cout << "\tscore = " << (Tree.empty () ? SequencesIter_lst.iterAlign () : SequencesIter_lst.iterAlign (Tree)) << endl;
			//cout << "ok" << endl;
		}
		if (ScoreConservedRes_b)
		{
			cout << "score conserved residues: ";
			scoreConservedRes (Sequences_lst);
			for (vector <C_AnalysedSeqs>::iterator i = PairSequences_vct.begin (); i != PairSequences_vct.end (); ++ i)
				scoreConservedRes (*i);
			cout << "ok" << endl;
		}
		if (ColorClusters_b)
		{
			cout << "color blocks: ";
			colorClusters (Sequences_lst);
			for (vector <C_AnalysedSeqs>::iterator i = PairSequences_vct.begin (); i != PairSequences_vct.end (); ++ i)
				colorClusters (*i);
			cout << "ok" << endl;
		}
		if (ColorClusterBreakers_b)
		{
			cout << "color block breakers: ";
			colorClusterBreakers (Sequences_lst);
			for (vector <C_AnalysedSeqs>::iterator i = PairSequences_vct.begin (); i != PairSequences_vct.end (); ++ i)
				colorClusterBreakers (*i);
			cout << "ok" << endl;
		}
		if (ColorAAs_b)
		{
			cout << "color amino acids: ";
			colorAAs (Sequences_lst);
			for (vector <C_AnalysedSeqs>::iterator i = PairSequences_vct.begin (); i != PairSequences_vct.end (); ++ i)
				colorAAs (*i);
			cout << "ok" << endl;
		}
		if (g_OutputFileName_str.empty ())
		{
			if (! Sequences_lst.empty ())
			{
				if (Sequences_lst.areAligned ())
					cout << "aligned sequences:" << endl << omsfseqs << Sequences_lst << endl;
								
				if (! PairSequences_vct.empty ())
				{
					cout << "pairwise aligned sequences to target:\n";
					for (vector <C_AnalysedSeqs>::const_iterator i = PairSequences_vct.begin (); i != PairSequences_vct.end (); ++ i)
					{
					  cout << omsfseqs << *i << endl << "---\n" << endl ;
					}
				}
			}
		}
		else if (!AlignPairwiseToTarget_b)
		{
			cout << "write sequences to out file '" << g_OutputFileName_str << "': ";
			C_OSeqFile OSeqFile (g_OutputFileName_str, g_OutputFileFormat_e);
			OSeqFile.start ();
			if (PairSequences_vct.empty () || Sequences_lst.areAligned ())
				OSeqFile.writeSequences (Sequences_lst);
			for (vector <C_AnalysedSeqs>::const_iterator i = PairSequences_vct.begin (); i != PairSequences_vct.end (); ++ i)
			{
				OSeqFile.writeSequences (*i);
			}
			OSeqFile.stop ();
			if (OSeqFile.fail ())
				cout << "failed" << endl;
			else
				cout << "ok" << endl;
		}
		if (! g_OutputParamFileName_str.empty ())
		{
			cout << "write parameters to out file '" << g_OutputParamFileName_str << "': ";
			C_ParameterFile OParamFile (g_OutputParamFileName_str.c_str (), ios::out);
			OParamFile.writeParameters ();
			if (OParamFile.fail ())
				cout << "failed" << endl;
			else
				cout << "ok" << endl;
		}
		
		cout << "exit." << endl;
		
	}
	catch (exception & e)
	{
		cout << "Error: " << e.what () << endl;
		exit (-1);
	}
	catch (...)
	{
		cout << "Error : unknown exception !" << endl;
		exit (-1);
	}
	
	return EXIT_SUCCESS;
}


void colorClusters (C_AnalysedSeqs & Sequences_lst)
{
	for (C_AnalysedSeqs::iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); i ++)
		i->colorClusters (g_DisplayParameters.clusterColors());
}


void colorClusterBreakers (C_AnalysedSeqs & Sequences_lst)
{
	for (C_AnalysedSeqs::iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); i ++)
		i->color_if <S_IsClusterBreaker> (g_DisplayParameters.clusterBreakersColor());
}


void colorAAs (C_AnalysedSeqs & Sequences_lst)
{
	for (C_AnalysedSeqs::iterator i = Sequences_lst.begin (); i != Sequences_lst.end (); i ++)
		for (C_DisplayParameters::aas_colors_t::const_iterator Color_i = g_DisplayParameters.aasColors ().begin (); Color_i != g_DisplayParameters.aasColors ().end (); Color_i ++)
			i->color_if (Color_i->first, Color_i->second);
}
