/***************************************************************************
                          s_analysedaa.h  -  description
                             -------------------
    begin                : Thu Jun 19 2003
    copyright            : (C) 2003 by Cyril Deremble
    email                : deremble@ihes.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef S_ANALYSEDAA_H
#define S_ANALYSEDAA_H


#include "_aminoacids.h"
#include "_aacodes.h"
#include "_xmltools.h"
#include "s_are.h"

namespace N_AminoAcid
{
using namespace N_XML;

const size_t CLUSTER_END = 0;
const size_t END_INCR = 1;
const size_t NO_ID = (size_t)-1;


/** structure used to manipulate sequences and clusters
 *@author Cyril Deremble
 */
struct S_AnalysedAA
{
    E_AminoAcid Value;

    size_t Prev;
    size_t Next;
    color_t Color;
    color_t ClusterColor;

    size_t ClusterId;
    size_t ConservedScore;

    S_AnalysedAA (E_AminoAcid aa = UNKNOWN,
                  size_t p = CLUSTER_END,
                  size_t n = CLUSTER_END,
                  color_t c = BLACK,
                  color_t cc = WHITE,
                  size_t ci = NO_ID,
                  size_t cs = 0)
            : Value (aa), Prev (p), Next (n), Color (c), ClusterColor (cc), ClusterId (ci), ConservedScore (cs)
    {}
    ;

    template <class T>
    S_AnalysedAA (const T & aa,
                  size_t p = CLUSTER_END,
                  size_t n = CLUSTER_END,
                  color_t c = BLACK,
                  color_t cc = WHITE,
                  size_t ci = NO_ID,
                  size_t cs = 0)
            : Prev (p), Next (n), Color (c), ClusterColor (cc), ClusterId (ci), ConservedScore (cs)
    {
        Value = convertAA <S_AnalysedAA> (aa);
    };

    operator E_AminoAcid () const
    {
        return (Value);
    };

    template <class T>
    operator T () const
    {
        return convertAA <T> (Value);
    };

    bool operator == (E_AminoAcid aa) const
    {
        return Value == aa;
    };
};

typedef S_AnalysedAA analysedAA_t;

inline void convertAA (const string & AA_str, S_AnalysedAA & aa)
{
    aa.Value = convertAA <E_AminoAcid> (AA_str);
};

inline void convertAA (const S_AnalysedAA & aa, E_AminoAcid & AA)
{
    AA = aa.Value;
};

inline void convertAA (const S_AnalysedAA & aa, char & c)
{
    convertAA (convertAA <E_AminoAcid> (aa), c);
};

inline void convert_AA (const S_AnalysedAA & aa, string & str)
{
    convertAA (convertAA <E_AminoAcid> (aa), str);
};

inline void convertAA (E_AminoAcid AA, S_AnalysedAA & aa)
{
    aa.Value = AA;
};

inline void convertAA (char AA_c, S_AnalysedAA & aa)
{
    aa.Value = convertAA <E_AminoAcid> (AA_c);
};


struct S_IsInCluster : public unary_function <S_AnalysedAA, bool>
{
    bool operator () (const S_AnalysedAA & AA) const
    {
        return (AA.Prev != CLUSTER_END) || (AA.Next != CLUSTER_END);
    };
};
const S_IsInCluster isInCluster = S_IsInCluster ();
const S_Are <S_IsInCluster> areInCluster = S_Are <S_IsInCluster> ();

struct S_IsFirstInCluster : public unary_function <S_AnalysedAA, bool>
{
    bool operator () (const S_AnalysedAA & AA) const
    {
        return (AA.Prev == CLUSTER_END) && (AA.Next != CLUSTER_END);
    };
};
const S_IsFirstInCluster isFirstInCluster = S_IsFirstInCluster ();
const S_Are <S_IsFirstInCluster> areFirstInCluster = S_Are <S_IsFirstInCluster> ();

struct S_IsLastInCluster : public unary_function <S_AnalysedAA, bool>
{
    bool operator () (const S_AnalysedAA & AA) const
    {
        return (AA.Prev != CLUSTER_END) && (AA.Next == CLUSTER_END);
    };
};
const S_IsLastInCluster isLastInCluster = S_IsLastInCluster ();
const S_Are <S_IsLastInCluster> areLastInCluster = S_Are <S_IsLastInCluster> ();

struct S_IsNotLastInCluster : public unary_function <S_AnalysedAA, bool>
{
    bool operator () (const S_AnalysedAA & AA) const
    {
        return (AA.Prev != CLUSTER_END);
    };
};
const S_IsNotLastInCluster isNotLastInCluster = S_IsNotLastInCluster ();
const S_Are <S_IsNotLastInCluster> areNotLastInCluster = S_Are <S_IsNotLastInCluster> ();


};

#endif

