<?xml version="1.0"?>

<!--phybal_parameters_stylesheet.xsl-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="html" indent="yes" encoding="UTF-8"/>

<!--/ template-->
<xsl:template match="/">
	<html>
		<body>
			<h1>Phybal parameters</h1>
			<hr/>
			<xsl:for-each select="//hcaal-parameters/*">
				<xsl:apply-templates select="."/>
			</xsl:for-each>
		</body>
	</html>
</xsl:template>


<!--align-parameters template-->
<xsl:template match="align-parameters">

	<h2>Sequence alignment parameters</h2>
	<ul>
		<xsl:for-each select="*">
			<xsl:apply-templates select="."/>
		</xsl:for-each>
	</ul>
	<hr/>
</xsl:template>

<!--gop template-->
<xsl:template match="gop">
	<li>Gap open penalty:
		<ul><li><xsl:value-of select="."/></li></ul>
	</li>
</xsl:template>

<!--gep template-->
<xsl:template match="gep">
	<li>Gap extend penalty: 
		<ul><li><xsl:value-of select="."/></li></ul>
	</li>
</xsl:template>

<!--end-gop template-->
<xsl:template match="end-gop">
	<li>Gap open penalty at the end of the sequences: 
		<ul><li><xsl:value-of select="."/></li></ul>
	</li>
</xsl:template>

<!--end-gep template-->
<xsl:template match="end-gep">
	<li>Gap extend penalty at the end of the sequences: 
		<ul><li><xsl:value-of select="."/></li></ul>
	</li>
</xsl:template>

<!--block-gop template-->
<xsl:template match="cluster-gop">
	<li>Gap open penalty when inside a block: 
		<ul><li><xsl:value-of select="."/></li></ul>
	</li>
</xsl:template>

<!--block-gep template-->
<xsl:template match="cluster-gep">
	<li>Gap extend penalty when inside a block: 
		<ul><li><xsl:value-of select="."/></li></ul>
	</li>
</xsl:template>

<!--score-matrix template-->
<xsl:template match="score-matrix">

	<li>Substitution matrix:
		<table style="text-align:center;" cellspacing="6"><xsl:copy-of select="."/></table>
	</li>
</xsl:template>

<!--score-matrix template-->
<xsl:template match="score-matrix[@in-cluster = 'true']">

	<li>Substitution matrix when inside a block:
		<table style="text-align:center" cellspacing="6"><xsl:copy-of select="."/></table>
	</li>
</xsl:template>

<!--hca-parameters template-->
<xsl:template match="hca-parameters">

	<h2>Hydrophobic Block detection parameters</h2>
	<ul>
		<xsl:for-each select="*">
			<xsl:apply-templates select="."/>
		</xsl:for-each>
	</ul>
	<hr/>
</xsl:template>

<!--block-neighbours template-->
<xsl:template match="cluster-neighbours">

	<li>Relative closest positions of the neighbours that can form a block with an amino acid:
		<ul>
			<xsl:for-each select="*">
				<li><xsl:value-of select="."/></li>
			</xsl:for-each>
		</ul>
	</li>
</xsl:template>

<!--aa-properties template-->
<xsl:template match="aa-properties">

	<h2>Amino acid properties</h2>
	<ul>
		<xsl:for-each select="*">
			<xsl:apply-templates select="."/>
		</xsl:for-each>
	</ul>
	<hr/>
</xsl:template>

<!--hydrophobic-aas template-->
<xsl:template match="hydrophobic-aas">

	<li>List of the hydrophobic amino acids:
		<ul>
			<xsl:for-each select="*">
				<li><xsl:value-of select="."/></li>
			</xsl:for-each>
		</ul>
	</li>
</xsl:template>

<!--block-breaker-aas template-->
<xsl:template match="cluster-breaker-aas">

	<li>List of the block breaker amino acids:
		<ul>
			<xsl:for-each select="*">
				<li><xsl:value-of select="."/></li>
			</xsl:for-each>
		</ul>
	</li>
</xsl:template>

<!--non-aas template-->
<xsl:template match="non-aas">

	<li>List of the symbols not considered as amino acids:
		<ul>
			<xsl:for-each select="*">
				<li><xsl:value-of select="."/></li>
			</xsl:for-each>
		</ul>
	</li>
</xsl:template>

<!--display-parameters template-->
<xsl:template match="display-parameters">

	<h2>Display parameters</h2>
	<ul>
		<xsl:for-each select="*">
			<xsl:apply-templates select="."/>
		</xsl:for-each>
	</ul>
	<hr/>
</xsl:template>

<!--block-colors template-->
<xsl:template match="cluster-colors">

	<li>List of the colors used to mark blocks:
		<ul>
			<xsl:for-each select="*">
				<li>
					<xsl:element name="font">
						<xsl:attribute name="style">background-color:<xsl:value-of select="."/>;</xsl:attribute>
						block
					</xsl:element>
				</li>
			</xsl:for-each>
		</ul>
	</li>
</xsl:template>

<!--block-breakers-color template-->
<xsl:template match="cluster-breakers-color">

	<li>Color used to mark block breakers:
		<xsl:element name="font">
			<xsl:attribute name="style">color:<xsl:value-of select="."/>;</xsl:attribute>
			block breaker
		</xsl:element>
	</li>
</xsl:template>

<!--colored-aas template-->
<xsl:template match="colored-aas">

	<li>List of the colored amino acids:
		<ul>
			<xsl:for-each select="*">
				<li>
					<xsl:element name="font">
						<xsl:attribute name="style">color:<xsl:value-of select="@color"/>;</xsl:attribute>
						<xsl:value-of select="."/>
					</xsl:element>
				</li>
			</xsl:for-each>
		</ul>
	</li>
</xsl:template>

<!--seq-line-width template-->
<xsl:template match="seq-line-width">

	<li>Number of amino acids per line:
		<ul><li><xsl:value-of select="."/></li></ul>
	</li>
</xsl:template>

<!--output-stylesheet-width template-->
<xsl:template match="output-stylesheet">

	<li>URL of the xsl stylesheet to transform the output xml file in html:
		<ul><li><xsl:value-of select="."/></li></ul>
	</li>
</xsl:template>

<!--aa-codes template-->
<xsl:template match="aa-codes">

	<h2>Amino acid codes used</h2>
	Phybal doesn't make any difference between uppercased and lowercased amino acid codes.
	<ul>
		<xsl:for-each select="*">
			<xsl:apply-templates select="."/>
		</xsl:for-each>
	</ul>
	<hr/>
</xsl:template>

<!--aa-code template-->
<xsl:template match="aa-code">

	<li><xsl:value-of select="@name"/> code:
		<table colspacing="10">
			<tr><th>Amino acid codes</th><th>Amino acid names</th></tr>
			<xsl:for-each select="*">
				<xsl:apply-templates select="."/>
			</xsl:for-each>
		</table>
		<br/>
	</li>
</xsl:template>

<!--pair template-->
<xsl:template match="pair">

	<tr>
		<td style="text-align:center;"><xsl:value-of select="@code"/></td>
		<td style="text-align:right;"><xsl:value-of select="@aa-name"/></td>
	</tr>
</xsl:template>

</xsl:stylesheet>
