<?xml version="1.0"?>

<!--hcaal_output_style.xsl-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="html" indent="yes" encoding="UTF-8"/>

<!--/ template-->
<xsl:template match="/">
	<html>
		<body>
			<xsl:for-each select="//sequences">
				<xsl:apply-templates select="."/>
			</xsl:for-each>
		</body>
	</html>
</xsl:template>


<!--sequences template-->
<xsl:template match="sequences">

	<xsl:element name="table">
		<xsl:attribute name="width">100%</xsl:attribute>
		<xsl:choose>
			<xsl:when test="@aligned = 'true'">
				<xsl:call-template name="aligned-sequences">
					<xsl:with-param name="width">
						<xsl:value-of select="@view-width"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:for-each select="*">
					<xsl:apply-templates select="."/>
					<tr>
						<td colspan="2">
							<hr/>
						</td>
					</tr>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:element>
</xsl:template>

<!--sequences template-->
<xsl:template name="aligned-sequences">

	<xsl:param name="begin">1</xsl:param>
	<xsl:param name="width"/>

	<xsl:for-each select="*">
		<xsl:apply-templates select="." mode="align">
			<xsl:with-param name="begin">
				<xsl:value-of select="$begin"/>
			</xsl:with-param>
			<xsl:with-param name="end">
				<xsl:value-of select="$begin + $width"/>
			</xsl:with-param>
		</xsl:apply-templates>
	</xsl:for-each>

	<xsl:if test="not ($begin + $width &gt; count (sequence[position () = '1']/aa))">
		<tr>
			<td></td><td style="text-align:center;"> <big>- - -</big> </td>
		</tr>
		<xsl:call-template name="aligned-sequences">
			<xsl:with-param name="begin">
				<xsl:value-of select="$begin + $width"/>
			</xsl:with-param>
			<xsl:with-param name="width">
				<xsl:value-of select="$width"/>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:if>
</xsl:template>


<!--sequence template-->
<xsl:template match="sequence">

	<tr>
		<td>
			<xsl:value-of select="./@name"/>
		</td>
		<td style="font-family:lucidatypewriter, monospace;">
			<xsl:for-each select="*">
				<xsl:apply-templates select="."/>
			</xsl:for-each>
		</td>
	</tr>
</xsl:template>

<!--sequence template-->
<xsl:template match="sequence" mode="align">

	<xsl:param name="begin"/>
	<xsl:param name="end"/>

	<tr>
		<td>
			<xsl:if test="$begin = 1">
				<xsl:value-of select="./@name"/>
			</xsl:if>
		</td>
		<td style="font-family:lucidatypewriter, monospace;">
			<xsl:for-each select="aa[position () &lt; $end]">
				<xsl:if test="not (position () &lt; $begin)">
					<xsl:apply-templates select="."/>
				</xsl:if>
			</xsl:for-each>
		</td>
	</tr>
</xsl:template>

<!--aa template-->
<xsl:template match="aa[@*]">

	<xsl:element name="font">
		<xsl:attribute name="style">
			<xsl:for-each select="./@*">
				<xsl:apply-templates select="."/>
			</xsl:for-each>
		</xsl:attribute>
		<xsl:value-of select="."/>
	</xsl:element>
</xsl:template>

<!--color template-->
<xsl:template match="@color">color:<xsl:value-of select="."/>;</xsl:template>

<!--cluster-color template-->
<xsl:template match="@cluster-color">background-color:<xsl:value-of select="."/>;</xsl:template>

<!--conserved-score template-->
<xsl:template match="@conserved-score">font-weight:<xsl:value-of select="."/>00;</xsl:template>

<!--cluster-id template-->
<xsl:template match="@cluster-id"/>

</xsl:stylesheet>
